from scipy.optimize.tnc import fmin_tnc
import scipy.optimize.tnc as tnc
import openopt
from openopt.kernel.setDefaultIterFuncs import *
from openopt.kernel.ooMisc import WholeRepr2LinConst
from openopt.kernel.baseSolver import baseSolver

class scipy_tnc(baseSolver):
    __name__ = 'scipy_tnc'
    __license__ = "BSD"
    __authors__ = "Stephen G. Nash"
    __alg__ = "undefined"
    __info__ = 'box-bounded NLP solver, can handle lb<=x<=ub constraints, some lb-ub coords can be +/- inf'
    __optionalDataThatCanBeHandled__ = ['lb', 'ub']
    __isIterPointAlwaysFeasible__ = lambda self, p: True

    def __init__(self): pass

    def __solver__(self, p):
        #WholeRepr2LinConst(p)#TODO: remove me
        bounds = []
        for i in range(p.n): bounds.append((p.lb[i], p.ub[i]))
        messages = 0#TODO: edit me

        maxfun=p.maxFunEvals
        if maxfun > 1e8:
            p.warn('tnc cannot handle maxFunEvals > 1e8, the value will be used')
            maxfun = int(1e8)

        xf, nfeval, rc = fmin_tnc(p.f, x0 = p.x0, fprime=p.df, args=(), approx_grad=0, bounds=bounds, messages=messages, maxfun=maxfun, ftol=p.ftol, xtol=p.xtol, pgtol=p.gtol)

        if rc in (tnc.INFEASIBLE, tnc.NOPROGRESS): istop = FAILED_WITH_UNIMPLEMENTED_OR_UNKNOWN_REASON
        elif rc == tnc.FCONVERGED: istop = SMALL_DELTA_F
        elif rc == tnc.XCONVERGED: istop = SMALL_DELTA_X
        elif rc == tnc.MAXFUN: istop = IS_MAX_FUN_EVALS_REACHED
        elif rc == tnc.LSFAIL: istop = IS_LINE_SEARCH_FAILED
        elif rc == tnc.CONSTANT: istop = IS_ALL_VARS_FIXED
        elif rc == tnc.LOCALMINIMUM: istop = SOLVED_WITH_UNIMPLEMENTED_OR_UNKNOWN_REASON
        else:
            #TODO: IMPLEMENT USERABORT
            p.err('unknown stop reason')
        msg = tnc.RCSTRINGS[rc]
        p.istop, p.msg = istop, msg
        p.xf = xf

##        INFEASIBLE   = -1 # Infeasible (low > up)
##        LOCALMINIMUM =  0 # Local minima reach (|pg| ~= 0)
##        FCONVERGED   =  1 # Converged (|f_n-f_(n-1)| ~= 0)
##        XCONVERGED   =  2 # Converged (|x_n-x_(n-1)| ~= 0)
##        MAXFUN       =  3 # Max. number of function evaluations reach
##        LSFAIL       =  4 # Linear search failed
##        CONSTANT     =  5 # All lower bounds are equal to the upper bounds
##        NOPROGRESS   =  6 # Unable to progress
##        USERABORT    =  7 # User requested end of minimization

##RCSTRINGS = {
##        INFEASIBLE   : "Infeasible (low > up)",
##        LOCALMINIMUM : "Local minima reach (|pg| ~= 0)",
##        FCONVERGED   : "Converged (|f_n-f_(n-1)| ~= 0)",
##        XCONVERGED   : "Converged (|x_n-x_(n-1)| ~= 0)",
##        MAXFUN       : "Max. number of function evaluations reach",
##        LSFAIL       : "Linear search failed",
##        CONSTANT     : "All lower bounds are equal to the upper bounds",
##        NOPROGRESS   : "Unable to progress",
##        USERABORT    : "User requested end of minimization"
##}














"""
Some non-linear functions have much more restricted dom than R^nVars.
For example F(x) = log(x); dom F = R+ = {x: x>0}

For optimization solvers it is wont to expect user-povided F(x) = nan if x is out of dom.

I can't inform how successfully OO-connected solvers
will handle a prob instance with restricted dom
because it seems to be too prob-specific

Still I can inform that ralg handles the problems rather well
provided in every point x from R^nVars at least one ineq constraint is active
(i.e. value constr[i](x) belongs to R+)

Note also that some solvers require x0 inside dom objFunc.
For ralg it doesn't matter.
"""

from numpy import *
from openopt import NLP

n = 100
an = arange(n) # array [0, 1, 2, ..., n-1]
x0 = n+15*(1+cos(an))

f = lambda x: (x**2).sum() + sqrt(x**3).sum() 
df = lambda x: 2*x + 1.5*x**0.5

lb = zeros(n)
solvers = ['ralg']
#solvers = ['ipopt']
for solver in solvers:
    p = NLP(f, x0, df=df, lb=lb, xtol = 1e-6, iprint = 50, maxIter = 10000, maxFunEvals = 1e8)
    #p.checkdf()
    r = p.solve(solver)
# expected r.xf = small values near zero



__docformat__ = "restructuredtext en"

from numpy import *
from openopt import LUNP

M, N = 1500, 150
C = empty((M,N))
d =  empty(M)

for j in xrange(M):
    d[j] = 1.5*N+80*sin(j)
    C[j] = 8*sin(4.0+arange(N)) + 15*cos(j)

lb = sin(arange(N))
ub = lb + 1
p = LUNP(C, d, lb=lb, ub=ub)

r = p.solve('lp:glpk', iprint = -1)

print 'f_opt:', r.ff
#print 'x_opt:', r.xf




__docformat__ = "restructuredtext en"

from numpy import *
from openopt import MILP

f = [1, 2, 3, 4, 5, 4, 2, 1]

# indexing starts from ZERO!
# while in native lpsolve-python wrapper from 1
# so if you used [5,8] for native lp_solve python binding
# you should use [4,7] instead
intVars = [4, 7]

lb = -1.5 * ones(8)
ub = 15 * ones(8)
A = zeros((5, 8))
b = zeros(5)
for i in xrange(5):
    for j in xrange(8):
        A[i,j] = -8+sin(8*i) + cos(15*j)
    b[i] = -150 + 80*sin(80*i)

p = MILP(f=f, lb=lb, ub=ub, A=A, b=b, intVars=intVars, goal='min')
r = p.solve('lpSolve')
#r = p.solve('glpk', iprint =-1)
#r = p.solve('cplex')

print('f_opt: %f' % r.ff) # 25.801450769161505
print('x_opt: %s' % r.xf) # [ 15. 10.15072538 -1.5 -1.5 -1.  -1.5 -1.5 15.]



from numpy import hstack, vstack, atleast_1d, cumsum, asarray, zeros,  ndarray,\
prod, ones, copy, nan, flatnonzero, array_equal, asanyarray#, int64
from nonOptMisc import scipyInstalled, Hstack, Vstack, Find, \
isspmatrix, SparseMatrixConstructor, DenseMatrixConstructor
import numpy as np
#from oologfcn import OpenOptException


try:
    # available since numpy 1.6.x
    from numpy import count_nonzero
except:
    count_nonzero = lambda elem: len(flatnonzero(asarray(elem)))

def setStartVectorAndTranslators(p):
    from FuncDesigner import _Stochastic, ooarray
    startPoint = p.x0
    #assert all(asarray([atleast_1d(val).ndim for val in startPoint.values()]) == 1)
    
    fixedVars, freeVars = None, None
    
    allVarsAreFree = p.freeVars is None
    
    if not allVarsAreFree:
        if not isinstance(p.freeVars,  (list, tuple, ndarray, set)):
            assert hasattr(p.freeVars, 'is_oovar')
            p.freeVars = [p.freeVars]
            freeVars = p.freeVars
        else:
            freeVars = list(p.freeVars)
        freeVars = getVars(freeVars)
        fixedVars = list(set(startPoint.keys()).difference(set(freeVars)))
        p.fixedVars = fixedVars
    elif p.fixedVars is not None:
        p.fixedVars.update(p._aux_fixedVars)
        if not isinstance(p.fixedVars,  (list, tuple, ndarray, set)):
            assert hasattr(p.fixedVars, 'is_oovar')
            p.fixedVars = [p.fixedVars]
            fixedVars = p.fixedVars
        else:
            fixedVars = list(p.fixedVars)
            p.fixedVars = fixedVars
        for elem in fixedVars:
            if type(elem) == ooarray:
                p._init_fixed_ooarrays.add(elem)
        fixedVars = getVars(fixedVars)
        freeVars = list(set(startPoint.keys()).difference(set(fixedVars)))
        p.freeVars = freeVars
    else:
        freeVars = list(startPoint.keys())
    
    if len(p._aux_fixedVars):
        p.freeVars = freeVars = [elem for elem in freeVars if elem not in p._aux_fixedVars]
        if fixedVars is not None:
            fixedVars = list(set(p._aux_fixedVars.keys()) | set(fixedVars))
        else:
            fixedVars = list(p._aux_fixedVars.keys())
        p.fixedVars = fixedVars
    
    nn = len(freeVars)
    for i in range(nn):
        v = freeVars[nn-1-i]
        if isinstance(startPoint[v], _Stochastic):# TODO: is _Stochastic reachable here in freeVars? exclude if not
            if fixedVars is None:
                p.fixedVars = fixedVars = [v]
            else:
                fixedVars.append(v)
            if freeVars is None:
                freeVars = p.freeVars = startPoint.keys()
            del freeVars[nn-1-i]

    # TODO: use ordered set instead
    freeVars.sort(key=lambda elem: elem._id)
#    fixedVars.sort()
    p._freeVarsList = freeVars 
    p._discreteVarsNumList = []
    p._discreteVarsList = []
    for i, v in enumerate(p._freeVarsList):
        if v.domain is not None:
            p._discreteVarsNumList.append(i)
            p._discreteVarsList.append(v)
    
    p._fixedVars = p.fixedVarsSet = set(fixedVars) if fixedVars is not None else set()
    p._freeVars = p.freeVarsSet = set(freeVars) if freeVars is not None else set()
        
    # point should be FuncDesigner point that currently is Python dict        
    # point2vector = lambda point: atleast_1d(hstack([asfarray(point[oov]) for oov in freeVars]))
    
    tmp = {}
    for oov in freeVars:
        val = startPoint[oov]
        # TODO: is _Stochastic reachable here in freeVars? exclude if not
        tmp[oov] = 1 if isinstance(val, _Stochastic) else asanyarray(val).size
    
    p._optVarSizes = tmp#dict([(oov, asarray(startPoint[oov]).size) for oov in freeVars])
    sizes_items = list(p._optVarSizes.items())
    sizes_items.sort(key=lambda elem:elem[0]._id)
    point2vector = lambda point: atleast_1d(hstack([(point[oov] if oov in point else zeros(sz)) for oov, sz in sizes_items]))
    # 2nd case can trigger from objective/constraints defined over some of opt oovars only
        
    vector_x0 = point2vector(startPoint)
    n = vector_x0.size
    p.n = n
    
    #oovar_sizes = [asarray(startPoint[elem]).size for elem in freeVars]
    # temporary walkaround for pypy
    oovar_sizes = [len(atleast_1d(startPoint[elem]).flatten()) for elem in freeVars]

#    for elem in freeVars:
#        print startPoint[elem]
#        if type(startPoint[elem]) == ndarray: 
#            print '----'
#            print type(startPoint[elem])
#            print startPoint[elem].size 
#            print len(startPoint[elem])

    oovar_indexes = cumsum([0] + oovar_sizes)
    
    # TODO: mb use oovarsIndDict here as well (as for derivatives?)
    from FuncDesigner import oopoint
    startDictData = []
    if fixedVars is not None:
        for v in p.probDep & p._fixedVars:
            val = startPoint.get(v, 'absent')
            if type(val) == str and val == 'absent':
                p.err('value for fixed variable %s is absent in start point' % v.name)
            startDictData.append((v, val))

    #vector2point = lambda x: oopoint(startDictData + [(oov, x[oovar_indexes[i]:oovar_indexes[i+1]]) for i, oov in enumerate(freeVars)])
    p._FDtranslator = {'prevX':nan}
    def vector2point(x): 
#        x = asarray(x)
#        if not str(x.dtype).startswith('float'):
#            x = asfarray(x)
        x = atleast_1d(x).copy()
        if array_equal(x, p._FDtranslator['prevX']):
            return p._FDtranslator['prevVal']
            
        # without copy() ipopt and probably others can replace it by noise after closing
#        r = oopoint(startDictData + \
#                    [(oov, x[oovar_indexes[i]:oovar_indexes[i+1]]) for i, oov in enumerate(freeVars)])
        r = startDictData
        tmp = [(oov, x[oovar_indexes[i]:oovar_indexes[i+1]] if oovar_indexes[i+1]-oovar_indexes[i]>1 else x[oovar_indexes[i]]) for i, oov in enumerate(freeVars)]
#        for i, oov in enumerate(freeVars):
#            #I, J = oovar_indexes[i], oovar_indexes[i+1]
#            #r.append((oov, x[I] if J - I == 1 else x[I:J]))
#            r.append((oov, x[oovar_indexes[i]:oovar_indexes[i+1]]))
        r = oopoint(r+tmp, skipArrayCast = True)
        r.maxDistributionSize = p.maxDistributionSize
        p._FDtranslator['prevVal'] = r 
        p._FDtranslator['prevX'] = copy(x)
        return r

    oovarsIndDict = dict([(oov, (oovar_indexes[i], oovar_indexes[i+1])) for i, oov in enumerate(freeVars)])

    def pointDerivative2array(pointDerivative, useSparse = 'auto',  func=None, point=None): 
        
        # useSparse can be True, False, 'auto'
        if not scipyInstalled and useSparse == 'auto':
            useSparse = False
        if useSparse is True and not scipyInstalled:
            p.err('to handle sparse matrices you should have module scipy installed') 

        if len(pointDerivative) == 0: 
            if func is not None:
                funcLen = func(point).size
                if useSparse is not False:
                    return SparseMatrixConstructor((funcLen, n))
                else:
                    return DenseMatrixConstructor((funcLen, n))
            else:
                p.err('unclear error, maybe you have constraint independend on any optimization variables') 

        Items = pointDerivative.items()
        key, val = Items[0] if type(Items) == list else next(iter(Items))
        
        if isinstance(val, float) or (isinstance(val, ndarray) and val.shape == ()):
            val = atleast_1d(val)
        var_inds = oovarsIndDict[key]
        # val.size works in other way (as nnz) for scipy.sparse matrices
        funcLen, _rest = divmod(prod(val.shape), var_inds[1] - var_inds[0])
        assert _rest == 0, 'bug in openopt kernel'
        
        # CHANGES
        
        # 1. Calculate number of zero/nonzero elements
        involveSparse = useSparse
        if useSparse == 'auto':
            nTotal = n * funcLen#sum([prod(elem.shape) for elem in pointDerivative.values()])
            nNonZero = sum((elem.size if isspmatrix(elem) else count_nonzero(elem)) for elem in pointDerivative.values())
            involveSparse = 4*nNonZero < nTotal and nTotal > 1000
        if involveSparse:
            r2 = []
            if funcLen == 1:
                inds = []
                for oov, val in pointDerivative.items():
                    ind_start, ind_end = oovarsIndDict[oov]
                    
                    # works faster than isscalar()
                    if type(val) in (float, np.float64)\
                    or np.isscalar(val):
                        r2.append(val)
                        inds.append(ind_start)
                    elif type(val) in (np.ndarray, np.matrix):
                        Val = (val if type(val) == ndarray else val.A).flatten()
#                        if Val.size == 1:
#                            r2.append(Val.item())
#                            inds.append(ind_start)
#                        else:
                        Ind = np.where(Val)[0]
                        r2 += Val[Ind].tolist()
                        inds += (ind_start+Ind).tolist()
                    elif isspmatrix(val):
                        I, J, vals = Find(val)
#                        if vals.size == 1:
#                            r2.append(vals.item())
#                            inds.append(ind_start+J.item())
#                        else:
                        r2 += vals.tolist()
                        inds += (ind_start+J).tolist()

                from scipy.sparse import coo_matrix
                r3 = coo_matrix((r2, ([0]*len(r2), inds)), shape=(funcLen, n))
            else:
                # USE STACK
                ind_Z = 0
                derivative_items = list(pointDerivative.items())
                derivative_items.sort(key=lambda elem: elem[0]._id)
                for oov, val in derivative_items:#pointDerivative.items():
                    ind_start, ind_end = oovarsIndDict[oov]
                    if ind_start != ind_Z:
                        r2.append(SparseMatrixConstructor((funcLen, ind_start - ind_Z)))
                    if not isspmatrix(val): 
                        val = asarray(val) # else bug with scipy sparse hstack
                    r2.append(val)
                    ind_Z = ind_end
                if ind_Z != n:
                    # assert ind_Z < n
                    r2.append(SparseMatrixConstructor((funcLen, n - ind_Z)))
                r3 = Hstack(r2) 
                #if isspmatrix(r3) and 4 * r3.nnz > asarray(r3.shape, int64).prod(): r3 = r3.A
            return r3
        else:
            # USE INSERT
            if funcLen == 1:
                r = DenseMatrixConstructor(n)
            else:
                r = SparseMatrixConstructor((funcLen, n)) if involveSparse else DenseMatrixConstructor((funcLen, n)) 
            
            for key, val in pointDerivative.items():
                indexes = oovarsIndDict[key]
                if not involveSparse and isspmatrix(val): val = val.A
#                if isscalar(val) or prod(val.shape)==1:
#                    r[indexes[0]] = val.flatten() if type(val) == ndarray else val
#                el
                if r.ndim == 1:
                    r[indexes[0]:indexes[1]] = val.flatten() if type(val) == ndarray else val
                else:
                    r[:, indexes[0]:indexes[1]] = val if val.shape == r.shape else val.reshape((funcLen, prod(val.shape)/funcLen))
            # TODO: mb remove it
            if useSparse is True and funcLen == 1: 
                return SparseMatrixConstructor(r)
            elif r.ndim <= 1:
                r = r.reshape(1, -1)
            if useSparse is False and hasattr(r, 'toarray'):
                r = r.toarray()
            return r
                

    def getPattern(oofuns):
        # oofuns is Python list of oofuns
        assert isinstance(oofuns, list), 'oofuns should be Python list, inform developers of the bug'
        R = []
        for oof in oofuns:
            SIZE = asarray(oof(startPoint)).size
            r = []
            dep = oof._getDep()
            if len(p._fixedVars) != 0:
                dep = dep & p._freeVars if len(p._freeVars) < len(p._fixedVars) else dep.difference(p._fixedVars)
            
            # NEW
            ind_Z = 0
            vars = list(dep)
            vars.sort(key=lambda elem: elem._id)
            for oov in vars:
                ind_start, ind_end = oovarsIndDict[oov]
                if ind_start != ind_Z:
                    r.append(SparseMatrixConstructor((SIZE, ind_start - ind_Z)))
                r.append(ones((SIZE, ind_end - ind_start)))
                ind_Z = ind_end
            if ind_Z != n:
                # assert ind_Z < n
                r.append(SparseMatrixConstructor((SIZE, n - ind_Z)))
            if any([isspmatrix(elem) for elem in r]):
                rr = Hstack(r) if len(r) > 1 else r[0]
            elif len(r)>1:
                rr = hstack(r)
            else:
                rr = r[0]
            R.append(rr)
        result = Vstack(R) if any([isspmatrix(_r) for _r in R]) else vstack(R)
        
        return result
        
    p._getPattern = getPattern
    p.freeVars, p.fixedVars = freeVars, fixedVars
    p._point2vector, p._vector2point = point2vector, vector2point
    p._pointDerivative2array = pointDerivative2array
    p._oovarsIndDict = oovarsIndDict
    
    # TODO: replave p.x0 in RunProbSolver finish  
    p._x0, p.x0 = p.x0, vector_x0 
    
    def linearOOFunsToMatrices(oofuns): #, useSparse = 'auto'
        # oofuns should be linear
        C, d = [], []
        Z = p._vector2point(zeros(p.n))
        for elem in oofuns:
            if elem.isConstraint:
                lin_oofun = elem.oofun
            else:
                lin_oofun = elem
            if lin_oofun.getOrder(p.freeVars, p.fixedVars) > 1:
                from oologfcn import OpenOptException
                raise OpenOptException("this function hasn't been intended to work with nonlinear FuncDesigner oofuns")
            C.append(p._pointDerivative2array(lin_oofun.D(Z, **p._D_kwargs), useSparse = p.useSparse))
            d.append(-lin_oofun(Z))

        C, d = Vstack(C), hstack(d).flatten()

        return C, d    
    p._linearOOFunsToMatrices = linearOOFunsToMatrices





def getVars(t):
    from FuncDesigner import ooarray
    vars1 = [v for v in (t if t is not None else []) if type(v) != ooarray]
    vars2 = [v for v in (t if t is not None else []) if type(v) == ooarray]
    t = vars1 
    for elem in vars2:
        t += elem.tolist()
    return t



# Problem assignment in OpenOpt is performed in the following way:
from openopt import NLP 
# or other constructor names: LP, MILP, QP etc, 
# for full list see http://openopt.org/Problems
# p = NLP(*args, **kwargs)

"""
you should read help(NLP) for more details, 
also reading /examples/nlp_1.py and other files from the directory is highly recommended

Each class has some expected arguments
e.g. for NLP it's f and x0 - objective function and start point
thus using NLP(myFunc, myStartPoint) will assign myFunc to f and myStartPoint to x0 prob fields

alternatively, you could use it as kwargs, possibly along with some other kwargs:
"""

p = NLP(x0=15, f = lambda x: x**2-0.4, df = lambda x: 2*x, iprint = 0, plot = 1)

# after the problem is assigned, you could turn the parameters, 
# along with some other that have been set as defaults:

p.x0 = 0.15
p.plot = 0

def f(x):
    return x if x>0 else x**2
p.f = f

# At last, you can modify any prob parameters in minimize/maximize/solve/manage functions:

r = p.minimize('ralg', x0 = -1.5,  iprint = -1, plot = 1, color = 'r') 
# or
#r = p.manage('ralg', start = False, iprint = 0, x0 = -1.5)

"""
Note that *any* kwarg passed to constructor will be assigned
e.g. 
p = NLP(f, x0, myName='JohnSmith')
is equivalent to 
p.myName='JohnSmith'
It can be very convenient for user-supplied callback functions 
(see /examples/userCallback.py)
(instead of using "global" as you have to do in MATLAB)

See also http://openopt.org/OOFrameworkDoc#Result_structure for result structure (r) fields 
"""



#!/usr/bin/env python

from numpy import inf, copy, floor, log10, asfarray, asscalar, argsort

TkinterIsInstalled = True
try:
    from Tkinter import Tk, Button, Entry, Label, Frame, StringVar, DISABLED, END, IntVar, \
    Radiobutton, Canvas
    from tkFileDialog import asksaveasfilename, askopenfile 
    from tkMessageBox import showerror
except ImportError:
    TkinterIsInstalled = False

xtolScaleFactor = 1e-5

class mfa:
    filename = None # Python pickle file where to save results
    x0 = None
    solved = False
    def startSession(self):
        assert TkinterIsInstalled, '''
        Tkinter is not installed. 
        If you have Linux you could try using 
        "apt-get install python-tk"'''
        try:
            import nlopt
        except ImportError:
            s = '''
            To use OpenOpt multifactor analysis tool 
            you should have nlopt with its Python API installed,
            see http://openopt.org/nlopt'''
            print(s)
            showerror('OpenOpt', s)
            raw_input()
            return
        import os
        hd = os.getenv("HOME")
        self.hd = hd
        root = Tk()
        self.root = root
        from openopt import __version__ as oover
        root.wm_title(' OpenOpt %s Multifactor analysis tool for experiment planning ' % oover)
        SessionSelectFrame = Frame(root)
        SessionSelectFrame.pack(side='top', padx=230, ipadx = 40, fill='x', expand=True)
        var = StringVar()
        var.set('asdf')
        Radiobutton(SessionSelectFrame, variable = var, text = 'New', indicatoron=0, \
                    command=lambda: (SessionSelectFrame.destroy(), self.create())).pack(side = 'top', fill='x', pady=5)
        Radiobutton(SessionSelectFrame, variable = var, text = 'Load', indicatoron=0,  \
                    command=lambda:self.load(SessionSelectFrame)).pack(side = 'top', fill='x', pady=5)
        
        root.protocol("WM_DELETE_WINDOW", self.exit)

        root.mainloop()
        self.exit()

    
    def create(self, S={}):
        root = self.root
        
#        #BackgroundFile = Image(file='/home/dmitrey/tmp_i/Backgd01.jpg')
#        bgfile = '/home/dmitrey/tmp_i/Backgd01.gif'
#        #bgfile = '/home/dmitrey/IP.png'
#        BackgroundFile = PhotoImage(file=bgfile)
#        #RootFrame.create_image(0, 0, image=BackgroundFile)
        RootFrame = Canvas(root)#, image=BackgroundFile)
#        RootFrame.create_image(0, 0, image=BackgroundFile)
#        RootFrame.image = BackgroundFile
        RootFrame.pack()
        
        self.NameEntriesList, self.LB_EntriesList, self.UB_EntriesList, self.TolEntriesList, self.ValueEntriesList = [], [], [], [], []
        self.calculated_points = S.get('calculated_points', [])

        # Title
        #root.wm_title(' FuncDesigner ' + fdversion + ' Manager')
        
        C = Canvas(root)
        
        """                                              Buttons                                               """
        Frame(RootFrame).pack(ipady=4)
        #Label(root, text=' FuncDesigner ' + fdversion + ' ').pack()


        #                                                   Upper Frame
        UpperFrame = Frame(RootFrame)
        
        
        ProjectNameFrame = Frame(UpperFrame)#, relief = 'ridge', bd=2)
        Label(ProjectNameFrame,  text = 'Project name:').pack(side = 'left')
        ProjectNameEntry = Entry(ProjectNameFrame)
        ProjectNameEntry.pack(side = 'left')
        self.ProjectNameEntry = ProjectNameEntry
        ProjectNameFrame.pack(side = 'left')
        
        GoalSelectFrame = Frame(UpperFrame, relief = 'ridge', bd=2)
        GoalSelectText = StringVar(value = 'Goal:')
        Label(GoalSelectFrame, textvariable = GoalSelectText).pack(side = 'left')
        goal = StringVar()
        r1 = Radiobutton(GoalSelectFrame, text = 'Minimum', value = 'min', variable=goal)
        r1.pack(side = 'left')
        r2 = Radiobutton(GoalSelectFrame, text = 'Maximum', value = 'max', variable=goal)
        r2.pack(side = 'left')
        goal.set('min')    
        GoalSelectFrame.pack(side = 'left', padx = 10)
        self.goal = goal
        
        ObjectiveToleranceFrame = Frame(UpperFrame, relief = 'ridge', bd=2)
        ObjectiveToleranceFrame.pack(side='left')
        Label(ObjectiveToleranceFrame, text='Objective function tolerance:').pack(side = 'left')
        ObjTolEntry = Entry(ObjectiveToleranceFrame)
        ObjTolEntry.pack(side='left')
        self.ObjTolEntry = ObjTolEntry
        
        UpperFrame.pack(side = 'top', expand=True, fill = 'x')
        
        #                                                   Variables Frame
        varsRoot = Frame(RootFrame)
       
       
        #                                                    Lower frame
        LowerFrame = Frame(varsRoot)
        LowerFrame.pack(side = 'bottom', expand=True, fill = 'x')

        from webbrowser import open_new_tab
        About = Button(LowerFrame, text = 'About', command = lambda: open_new_tab('http://openopt.org/MultiFactorAnalysis'))
        About.pack(side='left')
        
        SaveButton = Button(LowerFrame, text = 'Save', command = self.save)
        SaveButton.pack(side='left', padx = 15)
        SaveAsButton = Button(LowerFrame, text = 'Save As ...', command = self.save)
        SaveAsButton.pack(side='left')
        Write_xls_Button = Button(LowerFrame, text = 'Write xls report', command = self.write_xls_report)
        Write_xls_Button.pack(side='left', padx = 15)
        
        
        
       
    #    PlotButton = Button(LowerFrame, text = 'Plot', command = lambda: Plot(C, self.prob))
    #    PlotButton.pack(side='left')
     
        ExperimentNumber = IntVar()
        ExperimentNumber.set(1)
        self.ExperimentNumber = ExperimentNumber
       
        ObjVal = StringVar()
        ObjEntry = Entry(LowerFrame, textvariable = ObjVal)
        self.ObjEntry = ObjEntry

        NN = StringVar(LowerFrame)
        NN_Label = Label(LowerFrame, textvariable = NN)
        
        
        names, lbs, ubs, tols, currValues = \
        Frame(varsRoot), Frame(varsRoot), Frame(varsRoot), Frame(varsRoot), Frame(varsRoot)
        Label(names, text=' Variable Name ').pack(side = 'top')
        Label(lbs, text=' Lower Bound ').pack(side = 'top')
        Label(ubs, text=' Upper Bound ').pack(side = 'top')
        Label(tols, text=' Tolerance ').pack(side = 'top')
        
        ValsColumnName = StringVar()
        ValsColumnName.set(' Initial Point ')
        Label(currValues, textvariable=ValsColumnName).pack(side = 'top')
        self.ValsColumnName = ValsColumnName
        
        
        #                                                    Commands Frame
        CommandsRoot = Frame(RootFrame)
        CommandsRoot.pack(side = 'right', expand = False, fill='y')
        
       
        AddVar = Button(CommandsRoot, text = 'Add Variable', command = \
                        lambda: self.addVar(names, lbs, ubs, tols, currValues))
        AddVar.pack(side = 'top', fill='x')

        Next = Button(CommandsRoot, text = 'Next', command = lambda: ExperimentNumber.set(ExperimentNumber.get()+1))
        #Next.pack(side='bottom',  fill='x')

        names.pack(side = 'left', ipady=5)
        lbs.pack(side = 'left', ipady=5)
        ubs.pack(side = 'left', ipady=5)
        tols.pack(side = 'left', ipady=5)
        currValues.pack(side = 'left', ipady=5)
        #currValues.pack_forget()
        
        varsRoot.pack()
        
        Start = Button(CommandsRoot, text = 'Start', \
                       command = lambda: (Start.destroy(), \
                                          Next.pack(side='bottom',  fill='x'), 
                                          #C.pack(side = 'bottom', expand=True, fill='both'), 
                                          r1.config(state=DISABLED), 
                                          r2.config(state=DISABLED), 
                                          ObjTolEntry.config(state=DISABLED), 
                                          ObjEntry.pack(side='right', ipady=4),
                                          NN_Label.pack(side='right'), \
                                          self.startOptimization(root, varsRoot, AddVar, currValues, ValsColumnName, ObjEntry, ExperimentNumber, Next, NN, 
                                                            goal.get(), float(ObjTolEntry.get()), C)))
        Start.pack(side = 'bottom', fill='x')
        self.Start = Start
        
        if len(S) != 0:
            for i in range(len(S['names'])):
                tmp = S['values'][i] if self.x0 is None else self.x0.split(' ')[i]
                self.addVar(names, lbs, ubs, tols, currValues, S['names'][i], S['lbs'][i], S['ubs'][i], S['tols'][i], tmp)
        else:
            self.addVar(names, lbs, ubs, tols, currValues)
#        for i in range(nVars):
#            self.addVar(names, lbs, ubs, tols, currValues)
        
        

    def addVar(self, names, lbs, ubs, tols, currValues, _name='', _lb='', _ub='', _tol='', _val=''):
        nameEntry, lb, ub, tol, valEntry = Entry(names), Entry(lbs), Entry(ubs), Entry(tols), Entry(currValues)
        nameEntry.insert(0, _name)
        lb.insert(0, _lb)
        ub.insert(0, _ub)
        tol.insert(0, _tol)
        valEntry.insert(0, _val)
        self.NameEntriesList.append(nameEntry)
        self.LB_EntriesList.append(lb)
        self.UB_EntriesList.append(ub)
        self.TolEntriesList.append(tol)
        self.ValueEntriesList.append(valEntry)
        nameEntry.pack(side = 'top')
        lb.pack(side = 'top')
        ub.pack(side = 'top')
        tol.pack(side = 'top')
        valEntry.pack(side = 'top')

    def startOptimization(self, root, varsRoot, AddVar, currValues, \
                          ValsColumnName, ObjEntry, ExperimentNumber, Next, NN, goal, objtol, C):
        AddVar.destroy()
        ValsColumnName.set('Experiment parameters')
        n = len(self.NameEntriesList)
        Names, Lb, Ub, Tol, x0 = [], [], [], [], []
        for i in range(n):
            N, L, U, T, valEntry = \
            self.NameEntriesList[i], self.LB_EntriesList[i], self.UB_EntriesList[i], self.TolEntriesList[i], self.ValueEntriesList[i]
            N.config(state=DISABLED)
            L.config(state=DISABLED)
            U.config(state=DISABLED)
            T.config(state=DISABLED)
            #valEntry.config(state=DISABLED)
            name, lb, ub, tol, val = N.get(), L.get(), U.get(), T.get(), valEntry.get()
            Names.append(name)
            x0.append(float(val))
            Lb.append(float(lb) if lb != '' else -inf)
            Ub.append(float(ub) if ub != '' else inf)
            
            # TODO: fix zero
            Tol.append(float(tol) if tol != '' else 0) 
            
        x0, Tol, Lb, Ub = asfarray(x0), asfarray(Tol), asfarray(Lb), asfarray(Ub)
        x0 *= xtolScaleFactor / Tol
        #self.x0 = copy(x0)
        from openopt import NLP, oosolver
        p = NLP(objective, x0, lb = Lb * xtolScaleFactor / Tol, ub=Ub * xtolScaleFactor / Tol)
        self.prob = p
        #calculated_points = [(copy(x0), copy(float(ObjEntry.get())))
        p.args = (Tol, self, ObjEntry, p, root, ExperimentNumber, Next, NN, objtol, C)
        #p.graphics.rate = -inf
        #p.f_iter = 2
        solver = oosolver('bobyqa', useStopByException = False)
        p.solve(solver, iprint = 1, goal = goal)#, plot=1, xlabel='nf')
        self.solved = True
        if p.stopcase >= 0:
            self.ValsColumnName.set('Best parameters')
            NN.set('Best obtained objective value:')
        #Next.config(state=DISABLED)
        Next.destroy()
        #reverse = True if goal == 'min' else False
        
        calculated_items = self.calculated_points.items() if isinstance(self.calculated_points, dict) else self.calculated_points
        vals = [calculated_items[i][1] for i in range(len(calculated_items))]
        ind = argsort(vals)
        j = ind[0] if goal == 'min' else ind[-1]
        key, val = calculated_items[j]
        text_coords = key.split(' ')
        for i in range(len(self.ValueEntriesList)):
            self.ValueEntriesList[i].delete(0, END)
            self.ValueEntriesList[i].insert(0, text_coords[i])
        ObjEntry.delete(0, END)
        obj_tol = self.ObjTolEntry.get()
        val = float(val) * 1e4 * objtol
        ObjEntry.insert(0, str(val))
        ObjEntry.config(state=DISABLED)
        #print('Finished')

    def Plot(C, p):
        pass
        #C.create_polygon()
    #    import os
    #    if os.fork():
    #        import pylab
    #        pylab.plot(p.iterValues.f)
    #        pylab.show()
    
    def load(self, SessionSelectFrame):
        file = askopenfile(defaultextension='.pck', initialdir = self.hd, filetypes = [('Python pickle files', '.pck')])
        if file in (None, ''):
            return
        SessionSelectFrame.destroy()
        import pickle 
        S = pickle.load(file)
        if type(S['calculated_points']) == dict: # for backward compatibility
            S['calculated_points'] = S['calculated_points'].items()
        #S['goal']='max'
        self.x0 = S.get('x0', None) # this line should be BEFORE self.create(S)
        
        self.create(S)
        
        self.ObjTolEntry.insert(0, S['ObjTol'])
        self.ProjectNameEntry.insert(0, S.get('ProjectName', ''))
        self.goal.set(S['goal'])
        self.ExperimentNumber.set(len(self.calculated_points))
        if len(S['calculated_points']) != 0: 
            self.Start.invoke()
        
    
    def save_as(self, filename=None):
        if filename is None:
            filename = asksaveasfilename(defaultextension='.pck', initialdir = self.hd, filetypes = [('Python pickle files', '.pck')])
        if filename in (None, ''):
            return
        if not self.solved and self.ObjEntry.get() != '':
            s = 'For the sake of more safety and some other circumstances saving with non-empty objective entry is forbidden'
            print(s)
            showerror('OpenOpt', s)
            return
        self.filename = filename
        names = [s.get() for s in self.NameEntriesList]
        lbs = [s.get() for s in self.LB_EntriesList]
        ubs = [s.get() for s in self.UB_EntriesList]
        tols = [s.get() for s in self.TolEntriesList]
        values = [s.get() for s in self.ValueEntriesList]
        goal = self.goal.get()
        ObjTol = self.ObjTolEntry.get()
        calculated_points = self.calculated_points
        ProjectName = self.ProjectNameEntry.get()
        S = {'names':names, 'lbs':lbs, 'ubs':ubs, 'tols':tols, 'values':values, 'goal':goal, \
        'ObjTol':ObjTol, 'calculated_points':calculated_points, 'ProjectName':ProjectName, 'x0':self.x0}
        
        # TODO: handle exceptions
        file = open(filename, "w")
        import pickle
        pickle.dump(S, file)
        file.close()
        
    save = lambda self: self.save_as(self.filename)    
    
    def write_xls_report(self):
        try:
            import xlwt
        except ImportError:
            s = '''To create xls reports 
            you should have xlwt installed, 
            see http://www.python-excel.org/
            you could use easy_install xlwt (with admin rights)
            also, in Linux you could use 
            [sudo] aptitude install python-xlwt
            '''
            print(s)
            showerror('OpenOpt', s)
            return
            
        xls_file = asksaveasfilename(defaultextension='.xls', initialdir = self.hd, filetypes = [('xls files', '.xls')])
        if xls_file in (None, ''):
            return
            
        wb = xlwt.Workbook()
        ws = wb.add_sheet('OpenOpt factor analysis report')
        ws.write(0, 0, 'Name')
        ws.write(0, 1, self.ProjectNameEntry.get())
        ws.write(1, 0, 'Goal')
        ws.write(1, 1, self.goal.get() + 'imum')
        ws.write(2, 0, 'Objective Tolerance')
        ws.write(2, 1, self.ObjTolEntry.get())
        
        names = [s.get() for s in self.NameEntriesList]
        lbs = [s.get() for s in self.LB_EntriesList]
        ubs = [s.get() for s in self.UB_EntriesList]
        tols = [s.get() for s in self.TolEntriesList]
        
        ws.write(4, 0, 'Variable')
        ws.write(5, 0, 'Lower Bound')
        ws.write(6, 0, 'Upper Bound')
        ws.write(7, 0, 'Tolerance')
        for i in range(len(names)):
            ws.write(4, i+1, names[i])
            ws.write(5, i+1, float(lbs[i]))
            ws.write(6, i+1, float(ubs[i]))
            ws.write(7, i+1, float(tols[i]))
        
        
        ws.write(9, 0, 'Exp Number')
        ws.write(9, len(names)+1, 'Objective')
        # TODO: minor code cleanup
        CP = self.calculated_points
        if isinstance(CP, dict):
            CP = CP.items()
        for i in range(len(CP)):
            key,  val = CP[i]
            ws.write(10+i, 0, i+1)
            coords = key.split()
            for j, coordVal in enumerate(coords):
                ws.write(10+i, j+1, float(coordVal))
            ws.write(10+i, len(coords)+1, float(val))
        wb.save(xls_file)
    
    def exit(self):
        try:
            self.root.quit()
        except:
            pass
        try:
            self.root.destroy()
        except:
            pass

def objective(x, Tol, mfa, ObjEntry, p, root, ExperimentNumber, Next, NN, objtol, C):
    #print 'in objective', x
    Key = ''
    Values = []
    ValueEntriesList = mfa.ValueEntriesList
    calculated_points = mfa.calculated_points
    for i in range(x.size):
        Format = '%0.9f' if Tol[i] == 0 else ('%0.' + ('%d' % (-floor(log10(Tol[i])))) + 'f') if Tol[i]<1 else '%d'
        tmp = x[i] * Tol[i] / xtolScaleFactor
        key = Format % tmp
        Key += key + ' '
        Values.append(key)
    #print calculated_points
    #print Key
    if mfa.x0 is None: mfa.x0 = Key
    if Key in dict(calculated_points): # TODO: rework it
        return dict(calculated_points)[Key]
   
    for i in range(x.size):
        ValueEntriesList[i].delete(0, END)
        ValueEntriesList[i].insert(0, Values[i])
    NN.set('Enter experiment %i result:' % int(len(calculated_points)+1))
    
    ObjEntry.delete(0, END)
    root.wait_variable(ExperimentNumber)
    r = float(ObjEntry.get()) 
    
#    from scipy import rand
#    r = abs(x[0]* Tol[0] / xtolScaleFactor-0.13) + abs(x[1]* Tol[1] /xtolScaleFactor-0.15) #+ 0.0001 * rand(1)

    r *= 1e-4 / objtol
    if isinstance(calculated_points, list):
        calculated_points.append((Key, asscalar(copy(r)))) 
    else: # dict
        calculated_points[Key] = asscalar(copy(r))
    
#    rr = []
#    for i, val in enumerate(p.iterValues.f):
#        rr.append(i)
#        rr.append(val)
#    rr.append(i+1)
#    rr.append(r)
#    if len(p.iterValues.f) > 1:
#        C.create_line(*rr, fill = 'blue')
    return r
    
MFA = lambda: mfa().startSession()
if __name__ == '__main__':
    MFA()
    



from numpy import asarray,  ones, all, isfinite, copy, nan, concatenate, array, asfarray, zeros
from openopt.kernel.ooMisc import WholeRepr2LinConst, xBounds2Matrix
from cvxopt_misc import *
import cvxopt.solvers as cvxopt_solvers
from cvxopt.base import matrix
from openopt.kernel.setDefaultIterFuncs import SOLVED_WITH_UNIMPLEMENTED_OR_UNKNOWN_REASON,  IS_MAX_ITER_REACHED, IS_MAX_TIME_REACHED, FAILED_WITH_UNIMPLEMENTED_OR_UNKNOWN_REASON, UNDEFINED

def converter_to_CVXOPT_SDP_Matrices_from_OO_SDP_Class(OO_SDP_Class_2D_Dict_S,  nVars):
    # nVars can be extracted from OO_SDP_Class_2D_Dict_S but it's easier just to pass as param
    a = OO_SDP_Class_2D_Dict_S
    #r = []
    R = {}
    for i, j in a.keys():
        if i not in R.keys():
            R[i] = zeros((nVars, asarray(a[i, 0]).size))
        R[i][j] = asfarray(a[i, j]).flatten()
    r = []
    for i in R.keys():
        r.append(Matrix(R[i].T))
    return r
            
def DictToList(d):
    i = 0
    r = []
    while i in d.keys():
        r.append(matrix(d[i], tc = 'd'))
        i += 1
    return r

def CVXOPT_SDP_Solver(p, solverName):
    if solverName == 'native_CVXOPT_SDP_Solver': solverName = None
    cvxopt_solvers.options['maxiters'] = p.maxIter
    cvxopt_solvers.options['feastol'] = p.contol    
    cvxopt_solvers.options['abstol'] = p.ftol
    if p.iprint <= 0:
        cvxopt_solvers.options['show_progress'] = False
        cvxopt_solvers.options['LPX_K_MSGLEV'] = 0
        #cvxopt_solvers.options['MSK_IPAR_LOG'] = 0
    xBounds2Matrix(p)

    #FIXME: if problem is search for MAXIMUM, not MINIMUM!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    f = copy(p.f).reshape(-1,1)

    # CVXOPT have some problems with x0 so currently I decided to avoid using the one

    sol = cvxopt_solvers.sdp(Matrix(p.f), Matrix(p.A), Matrix(p.b), \
                             converter_to_CVXOPT_SDP_Matrices_from_OO_SDP_Class(p.S, p.n), \
                             DictToList(p.d), Matrix(p.Aeq), Matrix(p.beq), solverName)
    p.msg = sol['status']
    if p.msg == 'optimal' :  p.istop = SOLVED_WITH_UNIMPLEMENTED_OR_UNKNOWN_REASON
    else: p.istop = -100
    if sol['x'] is not None:
        p.xf = asarray(sol['x']).flatten()
        p.ff = sum(p.dotmult(p.f, p.xf))
        #p.duals = concatenate((asarray(sol['y']).flatten(), asarray(sol['z']).flatten()))
    else:
        p.ff = nan
        p.xf = nan*ones(p.n)



__docformat__ = "restructuredtext en"
from numpy import isfinite, any, asarray
def check(p):
    """
    this func is called from runProbSolver(), you don't need to call the one
    """
    nErrors = 0

    if p.allowedGoals is not None and p.goal is not None and p.goal not in p.allowedGoals:
        # p.goal is None in EIG
        p.err('goal '+ p.goal+' is not available for the '+ p.probType + ' class (at least not implemented yet)')

    for fn in p._optionalData:
        attr = getattr(p, fn, None)
        if (attr is None or isinstance(attr, (list,tuple)) and len(attr)==0) or fn in p.solver.__optionalDataThatCanBeHandled__: continue
        if fn == 'Qc' or \
        (callable(attr) and getattr(p.userProvided, fn)) or \
        (type(attr) in (set, dict, list, tuple) and len(attr) != 0) or \
        (not callable(attr) and asarray(attr).size>0 and any(isfinite(attr))):
            p.err('the solver ' + p.solver.__name__ + ' cannot handle ' + "'" + fn + "' data")

    return nErrors




import numpy as np
import socket

class Wormhole:
    def __init__(self):
        TCP_IP = '127.0.0.1'
        TCP_PORT = 5002
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        s.bind((TCP_IP, TCP_PORT))
        s.listen(1)
#        print "waiting for connection"
        self.conn, addr = s.accept() # hangs until other end connects
#        print 'Connection address:', addr
        
    def put(self,name,val):
        val = np.atleast_2d(val)
        self.conn.send("%10s"%"put")
        self.conn.send("%10s"%name)
        self.conn.send("%30s"%str(val.shape).replace("(","[").replace(")","]"))
        n_floats = val.size
        n_sent = 0
        val_str = val.astype('>f8').tostring(order='F')
        while n_sent < n_floats:
            n_tosend = min(128,n_floats-n_sent)
            self.conn.send(val_str[8*n_sent:8*(n_sent+n_tosend)])
            n_sent += n_tosend
#            print "%i/%i floats sent"%(n_sent,n_floats)
            
    def execute(self,stmt):
        self.conn.send("%10s"%"exec")
        self.conn.send("%100s"%stmt)
        
    def get(self,name):
        self.conn.send("%10s"%"get")
        self.conn.send("%10s"%name)
        shape = tuple(map(int,self.conn.recv(30).split()))
        n_floats = np.prod(shape)
        val_flat = np.zeros(n_floats)        
        n_read = 0
        while n_read < n_floats:
            n_toread = min(128,n_floats-n_read)
            val_flat[n_read:n_read+n_toread] = np.fromstring(self.conn.recv(n_toread*8),'>f8')
            n_read += n_toread
#            print "%i/%i floats read"%(n_read,n_floats)
            
        return val_flat.reshape(shape,order="F")
            
            
            
        

        



from openopt import GLP
from numpy import *

f = lambda x: (x*x - 10*cos(2*pi*x) + 10).sum() #Rastrigin function
p = GLP(f, lb = -ones(10)*5.12,  ub = ones(10)*5.12,  maxIter = 1e3,  maxFunEvals = 1e5,  maxTime = 10,  maxCPUTime = 10)

r = p.solve('de', plot=0)
x_opt,  f_opt = r.xf,  r.ff
print x_opt



"""
The example illustrates oosolver usage
You should pay special attention for "isInstalled" field

oosolver work is untested for converters
"""
from openopt import oosolver, NLP

ipopt = oosolver('ipopt', color='r') # oosolver can hanlde prob parameters
ralg = oosolver('ralg', color='k', alp = 4.0) # as well as solver parameters
asdf = oosolver('asdf')

solvers = [ralg, asdf, ipopt]
# or just
# solvers = [oosolver('ipopt', color='r'), oosolver('asdf'), oosolver('ralg', color='k', alp = 4.0)]

for solver in solvers:
    if not solver.isInstalled:
        print 'solver ' + solver.__name__ + ' is not installed'
        continue
    p = NLP(x0 = 15, f = lambda x: x**4, df = lambda x: 4 * x**3, iprint = 0)
    r = p.solve(solver, plot=1, show = solver == solvers[-1])



from runProbSolver import runProbSolver
def ooRun(prob, solvers, *args, **kwargs):
    r = []
    for solver in solvers:
        r.append(runProbSolver(prob.copy(), solver, *args, **kwargs))
    return r



from openopt import GLP
from numpy import *

f = lambda x: (x[0]-1.5)**2 + sin(0.8 * x[1] ** 2 + 15)**4 + cos(0.8 * x[2] ** 2 + 15)**4 + (x[3]-7.5)**4
p = GLP(f, lb = -ones(4),  ub = ones(4),  maxIter = 1e3,  maxFunEvals = 1e5,  maxTime = 3,  maxCPUTime = 3)

#optional: graphic output
#p.plot = 1 or p.solve(..., plot=1) or p = GLP(..., plot=1)

r = p.solve('de', plot=1)
x_opt,  f_opt = r.xf,  r.ff



from numpy import hstack, ravel, isnan, asfarray, log10, array,  isfinite
from openopt import __version__ as ooversion
ooversion = str(ooversion)
from setDefaultIterFuncs import stopcase

class Graphics:
    def __init__(self):
        self.drawFuncs = [self.oodraw]
        self.specifierStart = 'd'
        self.specifierFailed = 'x'
        self.specifierOK = 'p'
        self.specifierUndefined = 'o'
        self.specifierContinueFeasible = 'v'
        self.specifierContinueInfeasible = '>'
        self.specifierError = 's'
        self.REDUCE = 1e8
        self.axLineStyle= '-'
        self.axLineWidth= 2

        self.axMarker = ''
        self.axMarkerSize = 1
        self.markerEdgeWidth = 1
        self.axMarkerEdgeColor = 'b'
        self.axFaceColor = 'y'

        # figure updating rate, (time elapsed for graphics) / (time passed)
        self.rate = 0.5

        self.drawingInOneWindow = True#some solvers for the same problem
        #what do you want on label x?
        #self.xlabel = 'time'#case-unsensitive
        #other values: CPUTime, iter
        #ignores time, spent on figure updatings
        #iter not recomended because iterations of different solvers take
        #different time
        #cputime is unrecomended on computers with several CPU
        #because some solvers can handle different number of CPU units
        #so time is best provided no other programs consume much cputime

        self.markerSize = 12
        self.iterMarkerSize = 1
        self.plotIterMarkers = True
        #self.plotOnlyCurrentMinimum = 0


    def oodraw(self, p): #note that self is same as p.graphics
        try:
            import pylab
        except:
            p.pWarn('to use OpenOpt graphics you need pylab (Python module) installed. Turning graphics off...')
            p.plot = 0
            return

        #isNewFigure = (not isempty(lastTask) and not isequal(lastTask, {p.name p.primal.fName})) ...
            #or (~self.drawingInOneWindow and globstat.iter ==1)...
            #or ~isfield(self, 'figureHandler')...
            #or (isequal(p.env, 'matlab') and ~ishandle(self.figureHandler))

        #todo: fix me later!
        needNewFigure = not p.iter

        #lastTask={p.name p.primal.fName} % x0 can be absent
        colors = ['b', 'k', 'c', 'r', 'g']
        specifiers = ['-', ':', '-.', '--']
        pylab.ion()
        if needNewFigure:

            #self.figureHandler = pylab.figure
            self.colorCount = -1
            self.specifierCount = 0
            self.nTrajectories = 0
            self.ghandlers = []
            #self.solverNames = []

            #TODO: the condition should be handled somewhere other place, not here.
            if p.probType in ('NLSP', 'SNLE'):
                Y_LABELS = ['maxResidual']
                #pylab.plot([0], [log10(p.contol)-1.5])
            elif p.probType == 'NLLSP':
                Y_LABELS = ['sum(residuals^2)']
            else:
                Y_LABELS = ['objective function']
            isIterPointAlwaysFeasible = p.solver.__isIterPointAlwaysFeasible__ if type(p.solver.__isIterPointAlwaysFeasible__) == bool \
                else p.solver.__isIterPointAlwaysFeasible__(p)
            if not (p._isUnconstrained() or isIterPointAlwaysFeasible):
                self.isMaxConstraintSubplotRequired = True
                if p.useScaledResidualOutput:
                    Y_LABELS.append('MaxConstraint/ConTol')
                else:
                    Y_LABELS.append('maxConstraint')
            else: self.isMaxConstraintSubplotRequired = False

            if self.isMaxConstraintSubplotRequired: self.nSubPlots = 2
            else: self.nSubPlots = 1


        #creating new trajectory, if needed
        isNewTrajectory = not p.iter # FIXME
        if isNewTrajectory:
            self.colorCount += 1
            if self.drawingInOneWindow:
                if self.colorCount > len(colors) - 1 :
                    self.colorCount = 0
                    self.specifierCount += 1
                    if self.specifierCount > len(specifiers) - 1 :
                        p.warn('line types number exeeded')
                        self.specifierCount = 0

        #setting color & specifier
        #color = colors[self.colorCount]
        #specifier = specifiers[self.specifierCount]
        color = p.color
        specifier = p.specifier


        #setting xlabel, ylabel, title etc
        tx = p.xlabel.lower()
        if isNewTrajectory:

            self.nTrajectories += 1

            #win = gtk.Window()
            #win.set_name("OpenOpt " + str(oover) + ", license: BSD 2.0")
            pTitle = 'problem: ' + p.name
            if p.showGoal: pTitle += '       goal: ' + p.goal
            if self.nSubPlots>1: pylab.subplot(self.nSubPlots, 1, 1)
            p.figure = pylab.gcf()
            pylab.title(pTitle)
            p.figure.canvas.set_window_title('OpenOpt ' + ooversion)

            if tx == 'cputime':
                xlabel = 'CPU Time elapsed (without graphic output), sec'
                d_x = 0.01
            elif tx == 'time':
                xlabel = 'Time elapsed (without graphic output), sec'
                d_x = 0.01
            elif tx in ['niter',  'iter']:
                xlabel = 'iter'
                d_x = 4
            elif tx == 'nf':
                xlabel = 'Number of objective function evaluations'
                d_x = 4
            else:  p.err('unknown graphic output xlabel: "' + tx + '", should be in "time", "cputime", "iter", "nf"')
            self.nPointsPlotted = 0

            for ind in range(self.nSubPlots):
                if (self.nSubPlots > 1 and ind != 0) or p.probType in ('NLSP', 'SNLE'):
                    ax = pylab.subplot(self.nSubPlots, 1, ind+1)
                    ax.set_yscale('log')

#                if self.nSubPlots > 1:
#                    ax = pylab.subplot(self.nSubPlots, 1, ind+1)
#                    if p.probType in ('NLSP', 'SNLE') or ind != 0:
#                        ax.set_yscale('log')
                pylab.hold(1)
                pylab.grid(1)
                pylab.ylabel(Y_LABELS[ind])

            pylab.xlabel(xlabel)


        ################ getting data to plot ###############
        if p.iter>0:
            IND_start, IND_end = self.nPointsPlotted-1, p.iter+1#note: indexing from zero assumed
            #if p.isFinished: IND_end = p.iter
        else: IND_start, IND_end = 0, 1

        if p.plotOnlyCurrentMinimum:
            yy = array(p.iterValues.f[IND_start:])
            if isNewTrajectory: self.currMin = yy[0]
            k = 0
            for j in range(IND_start, IND_start + len(yy)): #todo: min is slow in 1x1 comparison vs if-then-else
                yy[k] = min(self.currMin, p.iterValues.f[j])
                self.currMin = yy[k]
                k += 1
            if IND_start<=IND_end: 
                if len(p.iterValues.f) >= 1:
                    yySave = [p.iterValues.f[-1]] # FIXME! (for constraints)
                else:
                    yySave = [p.f(p.x0)]
        else: 
            yy = array(p.iterValues.f[IND_start:IND_end])
            if IND_start<=IND_end:
                if len(p.iterValues.f) >= 1: yySave = [p.iterValues.f[-1]]
                else:
                    yySave = [p.f(p.x0)]

        if tx == 'iter': xx = range(IND_start, IND_end)
        elif tx == 'cputime':
            if len(p.iterTime) != len(p.cpuTimeElapsedForPlotting): p.iterTime.append(p.iterTime[-1])
            xx = asfarray(p.iterCPUTime[IND_start:IND_end]) - asfarray(p.cpuTimeElapsedForPlotting[IND_start:IND_end])
        elif tx == 'time':
            if len(p.iterTime) != len(p.timeElapsedForPlotting): p.iterTime.append(p.iterTime[-1])
            xx = asfarray(p.iterTime[IND_start:IND_end]) - asfarray(p.timeElapsedForPlotting[IND_start:IND_end])
        elif tx == 'nf':
            xx = asfarray(p.iterValues.nf[IND_start:IND_end])
        else: p.err('unknown labelX case')

        if len(xx)>len(yy):
            if p.isFinished: xx = xx[:-1]
            else:  p.err('OpenOpt graphics ERROR - FIXME!')

        if p.probType in ('NLSP', 'SNLE'):
            yy = yy+p.ftol/self.REDUCE
        YY = [yy]
        #if len(YY) == 0: YY = yySave

        if self.isMaxConstraintSubplotRequired:
            yy22 = p.contol/self.REDUCE+asfarray(p.iterValues.r[IND_start:IND_end])
            if p.useScaledResidualOutput: yy22 /= p.contol
            YY.append(yy22)
            if IND_start<=IND_end:
                if len(p.iterValues.r) == 0: return
                rr = p.iterValues.r[-1] 
                if p.useScaledResidualOutput: rr /= p.contol
                if len(p.iterValues.r) >= 1: yySave.append(p.contol/self.REDUCE+asfarray(rr))
                else:
                    yySave.append(p.contol/self.REDUCE+asfarray(p.getMaxResidual(p.x0)))

        if needNewFigure:
            if self.nSubPlots > 1:
                pylab.subplot(2, 1, 2)
                tmp = 1 if p.useScaledResidualOutput else p.contol
                pylab.plot([xx[0]],[tmp / 10**1.5])
                pylab.plot([xx[0]+d_x],[tmp / 10**1.5])
                pylab.plot([xx[0]], [YY[1][0] * 10])
                pylab.plot([xx[0]+d_x], [YY[1][0] * 10])
                pylab.subplot(2, 1, 1)

            pylab.plot([xx[0]],[YY[0][0]])
            pylab.plot([xx[0]+d_x],[YY[0][0]])

        ##########################################
        if self.plotIterMarkers: usualMarker = 'o'
        else: usualMarker = ''

        for ind in range(self.nSubPlots):
            if self.nSubPlots > 1: pylab.subplot(self.nSubPlots,1,ind+1)
            yy2 = ravel(YY[ind])
            if len(yy2) < len(xx): 
                if IND_start > IND_end:
                    yy2 = ravel(yySave[ind])
                elif yy2.size == 0:
                    yy2 = ravel(yySave[ind])
                else:
                    yy2 = hstack((yy2, yy2[-1]))
#            if len(yy2)<len(xx):
#                if p.debug: p.warn('FIXME! - different len of xx and yy in graphics')
#                yy2 = yy2.tolist()+[yy2[-1]]
            if isNewTrajectory:
                if isfinite(p.xlim[0]): pylab.plot([p.xlim[0]],  [yy2[0]],  color='w')
                if isfinite(p.xlim[1]): pylab.plot([p.xlim[1]],  [yy2[0]],  color='w')
                if ind==0:
                    if isfinite(p.ylim[0]): pylab.plot([xx[0]],  [p.ylim[0]],  color='w')
                    if isfinite(p.ylim[1]): pylab.plot([xx[0]],  [p.ylim[1]],  color='w')
                    if p.probType in ('NLSP', 'SNLE'): pylab.plot([xx[0]], [p.ftol / self.REDUCE],  color='w')
            if ind == 1:
                horz_line_value = 1 if p.useScaledResidualOutput else p.primalConTol
                pylab.plot([xx[0], xx[-1]], [horz_line_value, horz_line_value], ls = self.axLineStyle, linewidth = self.axLineWidth, color='g',\
                marker = self.axMarker, ms = self.axMarkerSize, mew = self.markerEdgeWidth, mec = self.axMarkerEdgeColor, mfc = self.axFaceColor)
            elif p.probType in ('NLSP', 'SNLE'):
                pylab.plot([xx[0], xx[-1]], [p.ftol, p.ftol], ls = self.axLineStyle, linewidth = self.axLineWidth, color='g',\
                marker = self.axMarker, ms = self.axMarkerSize, mew = self.markerEdgeWidth, mec = self.axMarkerEdgeColor, mfc = self.axFaceColor)
            if isNewTrajectory:
                p2 = pylab.plot([xx[0]], [yy2[0]], color = color, marker = self.specifierStart,  markersize = self.markerSize)
                p3 = pylab.plot([xx[0], xx[0]+1e-50], [yy2[0], yy2[0]], color = color, markersize = self.markerSize)
                p._p3 = p3
                if p.legend == '': pylab.legend([p3[0]], [p.solver.__name__], shadow = True)
                elif type(p.legend) in (tuple, list): pylab.legend([p3[0]], p.legend, shadow = True)
                else: pylab.legend([p3[0]], [p.legend], shadow = True)
                pylab.plot(xx[1:], yy2[1:], color,  marker = usualMarker, markersize = self.markerSize/3)
            else:
                pylab.plot(xx, ravel(yy2), color + specifier, marker = usualMarker, markersize = self.iterMarkerSize)

            if p.isFinished:
                pylab.legend([p._p3[0]], [p.solver.__name__], shadow = True, loc=0)
                #xMin, xMax = [], []
                if p.istop<0:
                    if stopcase(p.istop) == 0: # maxTime, maxIter, maxCPUTime, maxFunEvals etc exeeded
                        if p.isFeas(p.xf):  s = self.specifierContinueFeasible
                        else: s = self.specifierContinueInfeasible
                    else: s = self.specifierFailed
                else:
                    #if not hasattr(p, 'isFeasible'): p.isFeas()
                    if p.isFeasible:
                    #if p.isFeas(p.xk)
                        if p.istop > 0:
                            s = self.specifierOK
                        else:# p.istop = 0
                            s = self.specifierUndefined
                    else: s = self.specifierError
                if s == self.specifierOK: marker = (5, 1, 0)
                else: marker = s
                if isnan(yy2[-1]):
                    yy2[-1] = 0
                    
                pylab.scatter(ravel(xx[-1]), [yy2[-1]], c=color, marker = marker, s=[150])

                #pylab.axis('auto')
                [xmin, xmax, ymin, ymax] = pylab.axis()
                if ymax - ymin > 25 * (yy2[-1] -ymin):
                    delta = 0.04 * (ymax - ymin)
                    pylab.scatter([(xmin+xmax)/2,  (xmin+xmax)/2], [ymin-delta, ymax+delta], s=1, c='w', edgecolors='none',  marker='o')
                    pylab.draw()
                if  ind == 0 and p.probType in ('NLSP', 'SNLE'):
                    pylab.plot([xmin, xmax], [log10(p.ftol), log10(p.ftol)],\
                        linewidth = self.axLineWidth, ls = self.axLineStyle, color='g',\
                        marker = self.axMarker, ms = self.axMarkerSize, \
                        mew = self.markerEdgeWidth, mec = self.axMarkerEdgeColor, mfc = self.axFaceColor)
                if  ind == 1:
                    horz_line_value = 0 if p.useScaledResidualOutput else log10(p.primalConTol)
                    pylab.plot([xmin, xmax], [horz_line_value, horz_line_value],\
                        linewidth = self.axLineWidth, ls = self.axLineStyle, color='g',\
                        marker = self.axMarker, ms = self.axMarkerSize, \
                        mew = self.markerEdgeWidth, mec = self.axMarkerEdgeColor, mfc = self.axFaceColor)
                    pylab.subplot(self.nSubPlots, 1, 1)
                    pylab.plot([xmax], [yy2[-1]],  color='w')
                    



#                if p.probType == 'NLSP':
#                    pylab.axhline(y=log10(p.primalConTol),  xmin=xmin, xmax=xmax,\
#                        linewidth = self.axLineWidth, ls = self.axLineStyle, color='g',\
#                        marker = self.axMarker, ms = self.axMarkerSize, \
#                        mew = self.markerEdgeWidth, mec = self.axMarkerEdgeColor, mfc = self.axFaceColor)


#        if p.isFinished:
#            for ind in range(self.nSubPlots):
#                if self.nSubPlots>1: pylab.subplot(self.nSubPlots,1,ind+1)
                #pylab.xlim(min(xMin), max(xMax))
        self.nPointsPlotted = p.iter+1
        pylab.draw()





from numpy.linalg import norm
from numpy import dot, asfarray, atleast_1d,  zeros, ones, int, float64, where, inf
from openopt.kernel.baseSolver import baseSolver

try:
    import bvls as BVLS
except:
    from openopt.kernel.oologfcn import OpenOptException
    raise OpenOptException('You should have bvls.f compiled via f2py, see OO LLSP doc webpage for details')

class bvls(baseSolver):
    __name__ = 'bvls'
    __license__ = "BSD"
    __authors__ = 'Robert L. Parker rlparker[at]ucsd.edu, Philip B. Stark stark[at]stat.berkeley.edu'
    __alg__ = '"Bounded Variable Least Squares:  An Algorithm and Applications" by P.B. Stark and R.L. Parker, in the journal "Computational Statistics", vol.10(2), 1995'
    __info__ = 'requires manual compilation of bvls.f by f2py, see OO online doc for details'
    __optionalDataThatCanBeHandled__ = ['lb', 'ub']

    __bvls_inf__ = 1e100
    T = float64
    def __init__(self): pass

    def __solver__(self, p):
        key = 0
        T = self.T
        istate = zeros(p.n+1,  int)
        m, n = p.C.shape[0],  p.n
        zz = zeros(m, T)
        act = zeros((m, m+2),  T)
        a, b = T(p.C),  T(p.d)
        bl,  bu = p.lb.copy(), p.ub.copy()
        bl[where(bl==-inf)[0]] = -self.__bvls_inf__
        bu[where(bu==inf)[0]] = self.__bvls_inf__

        xf, w, istop, msg,  iter = BVLS.bvls(key, a, b, bl, bu, p.maxIter, act, zz,  istate)

        p.istop, p.msg, p.iter = istop, msg.rstrip(), iter
        p.xf = xf
        #p.ff = p.fk = w[0]




from scipy.optimize import leastsq
from numpy import asfarray
from openopt.kernel.baseSolver import baseSolver


class scipy_leastsq(baseSolver):
    __name__ = 'scipy_leastsq'
    __license__ = "BSD"
    #__authors__ = 
    #__alg__ = ""
    __info__ = """
    MINPACK's lmdif and lmder algorithms
    """
    #__constraintsThatCannotBeHandled__ = (all)
    
    def __init__(self): pass
    def __solver__(self, p):
        
        p.xk = p.x0.copy()
        p.fk = asfarray((p.f(p.x0)) ** 2).sum().flatten()
            
        p.iterfcn()
        if p.istop:
            p.xf, p.ff = p.xk, p.fk
            return 
        
        if p.userProvided.df:
            xf, cov_x, infodict, mesg, ier = leastsq(p.f, p.x0, Dfun=p.df, xtol = p.xtol, ftol = p.ftol, maxfev = p.maxFunEvals, full_output = 1)
        else:
            xf, cov_x, infodict, mesg, ier = leastsq(p.f, p.x0, xtol = p.xtol, maxfev = p.maxFunEvals, epsfcn = p.diffInt, ftol = p.ftol, full_output = 1)
        
        if ier == 1: p.istop = 1000
        else: p.istop = -1000
        p.msg = mesg
            
        ff = asfarray((p.f(xf)) ** 2).sum().flatten()
        p.xk = xf
        p.fk = ff
        
        p.xf = xf
        p.ff = ff        
        p.iterfcn()
        





from openopt import QP
from numpy import  dot, asfarray, ones, zeros, max
from numpy.linalg import norm

def PolytopProjection(data, T = 1.0, isProduct = False, solver = None):
    if solver is None: 
        solver = 'cvxopt_qp'
        
    #data = float128(data)
    if isProduct:
        H = data
        n = data.shape[0]
        m = len(T)
    else:
        H = dot(data, data.T)
        n, m = data.shape
    #print H.shape
    #print 'PolytopProjection: n=%d, m=%d, H.shape[0]= %d, H.shape[1]= %d ' %(n, m, H.shape[0], H.shape[1])
    #T = abs(dot(H, ones(n)))
    f = -asfarray(T) *ones(n)
    p = QP(H, f, lb = zeros(n), iprint = -1, maxIter = 150)

    xtol = 1e-6
    if max(T) < 1e5*xtol: xtol = max(T)/1e5
    r = p._solve(solver, ftol = 1e-16, xtol = xtol, maxIter = 10000)
    sol = r.xf

    if isProduct:
        return r.xf
    else:
        s = dot(data.T, r.xf)
        return s.flatten(), r.xf        



from numpy import diag, array, sqrt,  eye, ones, inf, any, copy, zeros, dot, all, sum, isfinite, float64, isnan, log10, \
max, sign, array_equal, logical_and, matrix
from openopt.kernel.ooMisc import norm

try:
    from numpy.linalg import solve, LinAlgError
except ImportError:
    LinAlgError = Exception
    def solve(*args, **kw): 
        print('ralg with equality constraints is unimplemented yet')
        raise Exception('ralg with equality constraints is unimplemented yet')
from openopt.kernel.nonOptMisc import scipyAbsentMsg, scipyInstalled, isPyPy
import openopt

#try:
#    from numpy.linalg import cond
#except:
#    print 'warning: no cond in numpy.linalg, matrix B rejuvenation check will be omitted'
#    cond = lambda Matrix: 1

from openopt.kernel.baseSolver import *
#from openopt.kernel.Point import Point
from openopt.kernel.ooMisc import economyMult, Len
from openopt.kernel.setDefaultIterFuncs import *
from openopt.solvers.UkrOpt.UkrOptMisc import getBestPointAfterTurn
# for PyPy
from openopt.kernel.nonOptMisc import where

class ralg(baseSolver):
    __name__ = 'ralg'
    __license__ = "BSD"
    __authors__ = "Dmitrey"
    __alg__ = "Naum Z. Shor R-algorithm with adaptive space dilation & some modifications"
    __optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'c', 'h']
    iterfcnConnected = True
    _canHandleScipySparse = True

    #ralg default parameters
    B = None # if provided it should be square matrix of shape nVars x nVars
    alp, h0, nh, q1, q2  = 2.0, 1.0, 3, 'default:0.9 for NLP, 1.0 for NSP', 1.1
    hmult = 0.5
    S = 0
    T = float64
    dilationType = 'plain difference'

    showLS = False
    show_hs = False
    showRej = False
    showRes = False
    show_nnan = False
    doBackwardSearch = True
    approach = 'all active'
    newLinEq = True
    new_bs = True
    skipPrevIterNaNsInDilation = True
    innerState = None
    penalties = False # True means for nonlinear equalities only
    useLinePoints = True
    lineSearchFunc = None # used in some special problems
    #new_s = False

    def needRej(self, p, b, g, g_dilated):
#        r = log10(1e15 * p.norm(g_dilated) / p.norm(g))
#        if isfinite(r):
#            p.debugmsg('%d' % int(r))
        #p.debugmsg('%0.2g'%(p.norm(g_dilated) / p.norm(g)))
        return 1e14 * p.norm(g_dilated) < p.norm(g)
    #checkTurnByGradient = True

    def __init__(self): pass
    def __solver__(self, p):

        alp, h0, nh, q1, q2 = self.alp, self.h0, self.nh, self.q1, self.q2
        
        if isPyPy:
            if p.nc != 0 or p.nh != 0:
                p.warn("in PyPy ralg may work incorrectly with nonlinear constraints yet")
            if p.nbeq != 0 or any(p.lb==p.ub):
                p.err('in PyPy ralg cannot handle linear equality constraints yet')
        
        if type(q1) == str:
            if p.probType== 'NLP' and p.isUC: q1 = 0.9
            else: q1 = 1.0
        T = self.T
        # alternatively instead of alp=self.alp etc you can use directly self.alp etc

        n = p.n
        x0 = p.x0
        
        if p.nbeq == 0 or any(abs(p._get_AeqX_eq_Beq_residuals(x0))>p.contol): # TODO: add "or Aeqconstraints(x0) out of contol"
            x0[x0<p.lb] = p.lb[x0<p.lb]
            x0[x0>p.ub] = p.ub[x0>p.ub]
        
        ind_box_eq = where(p.lb==p.ub)[0]
        nEQ = ind_box_eq.size
        if nEQ != 0:
            initLenBeq = p.nbeq
            Aeq, beq, nbeq = copy(p.Aeq), copy(p.beq), p.nbeq
            p.Aeq = zeros([Len(p.beq) + nEQ, p.n])
            p.beq = zeros(Len(p.beq) + nEQ)
            p.beq[:Len(beq)] = beq
            p.Aeq[:Len(beq)] = Aeq
            for i in range(len(ind_box_eq)):
                p.Aeq[initLenBeq+i, ind_box_eq[i]] = 1
                p.beq[initLenBeq+i] = p.lb[ind_box_eq[i]] # = p.ub[indEQ[i]], because they are the same
            p.nbeq += nEQ
            
        if not self.newLinEq or p.nbeq == 0:
            #needProjection = False
            B0 = eye(n,  dtype=T)
            restoreProb = lambda *args: 0
            Aeq_r, beq_r, nbeq_r = None, None, 0
        else:
            #needProjection = True
            B0 = self.getPrimevalDilationMatrixWRTlinEqConstraints(p)
            #Aeq, beq, nbeq = p.Aeq, p.beq, p.nbeq
            
            if any(abs(p._get_AeqX_eq_Beq_residuals(x0))>p.contol/16.0):
                #p.debugmsg('old point Aeq residual:'+str(norm(dot(Aeq, x0)-beq)))
                try:
                    x0 = self.linEqProjection(x0, p.Aeq, p.beq)
                except LinAlgError:
                    s = 'Failed to obtain projection of start point to linear equality constraints subspace, probably the system is infeasible'
                    p.istop, p.msg = -25,  s
                    return
                    
                #p.debugmsg('new point Aeq residual:'+str(norm(dot(Aeq, x0)-beq)))
            if nEQ == 0:
                Aeq_r, beq_r, nbeq_r = p.Aeq, p.beq, p.nbeq
            else:
                Aeq_r, beq_r, nbeq_r = Aeq, beq, nbeq
            
            p.Aeq, p.beq, p.nbeq = None, None, 0
            
            # TODO: return prob with unmodified Aeq, beq
            
            def restoreProb():
                p.Aeq, p.beq, p.nbeq = Aeq_r, beq_r, nbeq_r
                #if nEQ != 0: restore lb, ub
                    
            
        b = B0.copy() if self.B is None else self.B
#        B_f = diag(ones(n))
#        B_constr = diag(ones(n))
        hs = asarray(h0, T)
        
        if self.innerState is not None:
            hs = self.innerState['hs']
            b = self.innerState['B']
        
        ls_arr = []
        w = asarray(1.0/alp-1.0, T)

        """                            Shor r-alg engine                           """
        bestPoint = p.point(array(copy(x0).tolist(), T)) # tolist() for PyPy compatibility
        prevIter_best_ls_point = bestPoint
        prevIter_PointForDilation = bestPoint

        g = bestPoint._getDirection(self.approach)
        prevDirectionForDilation = g
        moveDirection = g
        if not any(g) and all(isfinite(g)):
            # TODO: create ENUMs
            p.iterfcn(bestPoint)
            restoreProb()
            p.istop = 14 if bestPoint.isFeas(False) else -14
            p.msg = 'move direction has all-zero coords'
            return

        HS = []
        LS = []
        
        #SwitchEncountered = False
        selfNeedRej = False
        #doScale = False
        
        #directionVectorsList = []
#        #pass-by-ref! not copy!
#        if p.isFeas(p.x0): b = B_f
#        else: b = B_constr

#        if p.debug and hasattr(p, 'x_opt'):
#            import scipy
#            exactDirection = x0-p.x_opt
#            asdf_0 = exactDirection * (0.2+scipy.rand(n))
#            #asdf = asdf_0.copy()


        #fTol = p.fTol if p.fTol is not None else 15*p.ftol
        
        # CHANGES
        if self.penalties:
            oldVal = p.f(p.x0)
            newVal = inf
            x = p.x0
            
            #H,  DH = p.h, p.dh
            if p.nh != 0:
                #S = 1.0
            
                _Aeq = p.dh(x)
                _beq = -p.h(x)
                df = p.df(x)
                if n>=150 and not scipyInstalled:
                    p.pWarn(scipyAbsentMsg)
                if n>100 and scipyInstalled:
                    from scipy.sparse import eye as Eye # to prevent numpy.eye overwrite
                    HH = Eye(n, n)
                else:
                    HH = eye(n)
                qp = openopt.QP(H=HH, f=df, Aeq=_Aeq, beq=_beq)
                QPsolver = openopt.oosolver('cvxopt_qp', iprint=-1)
                if not QPsolver.isInstalled:
                    S = None
                else:
                    r = qp.solve(QPsolver)
                    S = 10.0*sum(abs(r.duals)) if r.istop > 0 else None
                
                while any(p.h(x)) > p.contol:
                    if S is not None:
                        p2 = getattr(openopt, p.probType)(p.f, x)
                        p.fill(p2)
                        p2.x0 = x
                        p2.h = p2.dh = None
                        p2.userProvided.h = p2.userProvided.dh = False
                        p2.nh = 0
                        p2.f = lambda *args, **kwargs: p.f(*args, **kwargs) + sum(abs(S * p.h(*args, **kwargs)))
                        p2.df = lambda *args, **kwargs: p.df(*args, **kwargs) + dot(S * sign(p.h(*args, **kwargs)), p.dh(*args, **kwargs))
                        #p2.iterfcn = p.iterfcn
    #                    def df2(*args, **kwargs):
    #                        r1 = p.df(*args, **kwargs)
    #                        r2 = S * dot(p.dh(*args, **kwargs).reshape(-1, 1), sign(p.h(*args, **kwargs))).flatten()
    #                        #raise 0
    #                        return r1+r2
    #                    #p2.df = lambda *args, **kwargs: p.df(*args, **kwargs) + S * dot(p.dh(x).reshape(-1, 1), sign(p.h(*args, **kwargs))).flatten()
    #                    p2.df = df2
    #                    #raise 0
                        r2 = p2.solve(p.solver, iprint=10)
                        if r2.stopcase >= 0:
                            x = r2.xf
                            p.solver.innerState = r2.extras['innerState']
                            oldVal, newVal = newVal, r2.ff
                        else:
                            if r2.istop == IS_LINE_SEARCH_FAILED:
                                # TODO: custom S as raising penalties
                                pass
                        
                        if p.isFeas(p2.xk):
                            p.xf = p.xk = p2.xk
                            p.istop, p.msg = p2.istop, p2.msg
                            return
                        else:
                            S *= 50
                            #print('max residual:%0.2e'% r2.rf)
                        
                    else: # failed to solve QP
                        break

        # CHANGES END

        """                           Ralg main cycle                                    """

        for itn in range(p.maxIter+10):
            doDilation = True
            lastPointOfSameType = None # to prevent possible bugs
            alp_addition = 0.0
            
            iterStartPoint = prevIter_best_ls_point
            x = iterStartPoint.x.copy()

            g_tmp = economyMult(b.T, moveDirection)
            if any(g_tmp): g_tmp /= p.norm(g_tmp)
            g1 = p.matmult(b, g_tmp)
            
#            norm_moveDirection = p.norm(g1)
#            if doScale:
#                g1 *= (norm_moveDirection_prev/norm_moveDirection) ** 0.5
#            norm_moveDirection_prev = norm_moveDirection

#            if p.debug and hasattr(p, 'x_opt'):
#                cos_phi_0 = p.matmult(moveDirection,  prevIter_best_ls_point.x - p.x_opt)/p.norm(moveDirection)/p.norm(prevIter_best_ls_point.x - p.x_opt)
#                cos_phi_1 = p.matmult(g1,  prevIter_best_ls_point.x - p.x_opt)/p.norm(g1)/p.norm(prevIter_best_ls_point.x - p.x_opt)
#                print('beforeDilation: %f  afterDilation: %f' % (cos_phi_0, cos_phi_1) )
#                asdf = asdf_0.copy()
#                g_tmp = economyMult(b.T, asdf)
#                
#                #g_tmp = p.matmult(b.T, asdf)
#                
#                if any(g_tmp): g_tmp /= p.norm(g_tmp)
#                asdf = p.matmult(b, g_tmp)
#                cos_phi = dot(asdf, exactDirection) / p.norm(asdf) / p.norm(exactDirection)
#                p.debugmsg('cos_phi:%f' % cos_phi)
#                assert cos_phi >0


            """                           Forward line search                          """

            hs_cumsum = 0
            hs_start = hs
            for ls in range(p.maxLineSearch):
                hs_mult = 1.0
                if ls > 20:
                    hs_mult = 2.0
                elif ls > 10:
                    hs_mult = 1.5
                elif ls > 2:
                    hs_mult = 1.05
                hs *= hs_mult

                x -= hs * g1
                hs_cumsum += hs

                newPoint = p.point(x) if ls == 0 else iterStartPoint.linePoint(hs_cumsum/(hs_cumsum-hs), oldPoint) #  TODO: take ls into account?
                
                if not p.isUC:
                    if newPoint.isFeas(True) == iterStartPoint.isFeas(True):
                        lastPointOfSameType = newPoint
              
                if self.show_nnan: p.info('ls: %d nnan: %d' % (ls, newPoint.__nnan__()))

                
                if ls == 0:
                    oldPoint = prevIter_best_ls_point#prevIterPoint
                    oldoldPoint = oldPoint
                    
                #if not self.checkTurnByGradient:
                if newPoint.betterThan(oldPoint, altLinInEq=True):
                    if newPoint.betterThan(bestPoint, altLinInEq=False): bestPoint = newPoint
                    oldoldPoint = oldPoint
                    oldPoint, newPoint = newPoint,  None
                else:
                    if not itn % 4: 
                        for fn in ['_lin_ineq', '_lin_eq']:
                            if hasattr(newPoint, fn): delattr(newPoint, fn)
                    break
                    
            hs /= hs_mult
            
            if ls == p.maxLineSearch-1:
                p.istop,  p.msg = IS_LINE_SEARCH_FAILED,  'maxLineSearch (' + str(p.maxLineSearch) + ') has been exceeded, the problem seems to be unbounded'
                restoreProb()
                return

            #iterPoint  = newPoint
            PointForDilation = newPoint
            #best_ls_point = newPoint if ls == 0 else oldPoint
            #if p.debug and ls != 0: assert not oldPoint.betterThan(best_ls_point)

            """                          Backward line search                          """
            mdx = max((150, 1.5*p.n))*p.xtol
            if itn == 0:  mdx = max((hs / 128.0, 128*p.xtol )) # TODO: set it after B rej as well
            ls_backward = 0
            maxLS = 3 if ls == 0 else 1
#            if ls <=3 or ls > 20:
            if self.doBackwardSearch:
                if self.new_bs:
                    best_ls_point,  PointForDilation, ls_backward = \
                    getBestPointAfterTurn(oldoldPoint, newPoint, maxLS = maxLS, maxDeltaF = 150*p.ftol, \
                                          maxDeltaX = mdx, altLinInEq = True, new_bs = True)
                    if PointForDilation.isFeas(True) == iterStartPoint.isFeas(True):
                        lastPointOfSameType = PointForDilation
#                        elif best_ls_point.isFeas(altLinInEq=True) == iterStartPoint.isFeas(altLinInEq=True):
#                            lastPointOfSameType = best_ls_point
                else:
                    best_ls_point, ls_backward = \
                    getBestPointAfterTurn(oldoldPoint, newPoint, maxLS = maxLS, altLinInEq = True, new_bs = False)
                    PointForDilation = best_ls_point

                # TODO: extract last point from backward search, that one is better than iterPoint
                if best_ls_point.betterThan(bestPoint): bestPoint = best_ls_point
                #p.debugmsg('ls_backward:%d' % ls_backward)
                if ls == 0 and ls_backward == -maxLS:
                    #pass
                    alp_addition += 0.25
                    #hs *= 0.9
                
                if ls_backward <= -1 and itn != 0:  # TODO: mb use -1 or 0 instead?
                    pass
                    #alp_addition -= 0.25*ls_backward # ls_backward less than zero
                
                #hs *= 2 ** min((ls_backward+1, 0))
            else:
                pass
                #hs *= 0.95
            
            best_ls_point = PointForDilation # elseware lots of difficulties
            
            """                                 Updating hs                                 """
            step_x = p.norm(PointForDilation.x - prevIter_PointForDilation.x)
            step_f = abs(PointForDilation.f() - prevIter_PointForDilation.f())
            HS.append(hs_start)
            assert ls >= 0
            LS.append(ls)
            if itn > 3:
                mean_ls = (3*LS[-1] + 2*LS[-2]+LS[-3]) / 6.0
                j0 = 3.3
                if mean_ls > j0:
                    hs = (mean_ls - j0 + 1)**0.5 * hs_start
                else:
                    #hs = (ls/j0) ** 0.5 * hs_start
                    hs = hs_start
                    if ls == 0 and ls_backward == -maxLS:
                        shift_x = step_x / p.xtol
                        RD = log10(shift_x+1e-100)
                        if PointForDilation.isFeas(True) or prevIter_PointForDilation.isFeas(True):
                            RD = min((RD, asscalar(asarray(log10(step_f / p.ftol + 1e-100)))))
                        if RD > 1.0:
                            mp = (0.5, (ls/j0) ** 0.5, 1 - 0.2*RD)
                            hs *= max(mp)
                            #from numpy import argmax
                            #print argmax(mp), mp

            """                            Handling iterPoints                            """
               
            best_ls_point = PointForDilation
            
            #if not SwitchEncountered and p.nh != 0 and PointForDilation.isFeas(altLinInEq=False) != prevIter_PointForDilation.isFeas(altLinInEq=False):
                #SwitchEncountered = True
                #selfNeedRej = True
            
            involve_lastPointOfSameType = False
            if lastPointOfSameType is not None and PointForDilation.isFeas(True) != prevIter_PointForDilation.isFeas(True):
                # TODO: add middle point for the case ls = 0
                assert self.dilationType == 'plain difference'
                #directionForDilation = lastPointOfSameType._getDirection(self.approach) 
                PointForDilation = lastPointOfSameType
                involve_lastPointOfSameType = True
                
            
           
           
            #directionForDilation = newPoint.__getDirection__(self.approach) # used for dilation direction obtaining
            
#            if not self.new_bs or ls != 0:
#                moveDirection = iterPoint.__getDirection__(self.approach)
#            else:
#                moveDirection = best_ls_point.__getDirection__(self.approach)
                
                #directionForDilation = pointForDilation.__getDirection__(self.approach) 
                
                
#                cos_phi = -p.matmult(moveDirection, prevIterPoint.__getDirection__(self.approach))
#                assert cos_phi.size == 1
#                if cos_phi> 0:
#                    g2 = moveDirection#pointForDilation.__getDirection__(self.approach) 
#                else:
#                    g2 = pointForDilation.__getDirection__(self.approach) 
                
            if itn == 0:
                p.debugmsg('hs: ' + str(hs))
                p.debugmsg('ls: ' + str(ls))
            if self.showLS: p.info('ls: ' + str(ls))
            if self.show_hs: p.info('hs: ' + str(hs))
            if self.show_nnan: p.info('nnan: ' + str(best_ls_point.__nnan__()))
            if self.showRes:
                r, fname, ind = best_ls_point.mr(True)
                p.info(fname+str(ind))

            """                         Set dilation direction                            """

            #if sum(p.dotmult(g, g2))>0:
                #p.debugmsg('ralg warning: slope angle less than pi/2. Mb dilation for the iter will be omitted.')
                #doDilation = False



                    
            # CHANGES
#            if lastPointOfSameType is None:
#                if currIterPointIsFeasible and not prevIterPointIsFeasible:
#                    alp_addition += 0.1
#                elif prevIterPointIsFeasible and not currIterPointIsFeasible:
#                    alp_addition -= 0.0
                
            # CHANGES END
            
#            r_p, ind_p, fname_p = prevIter_best_ls_point.mr(1)
#            r_, ind_, fname_ = PointForDilation.mr(1)


            #else:
            

            #print itn,'>>>>>>>>>', currIterPointIsFeasible
            
            """                                    Excluding derivatives switched to/from NaN                                    """
            
            if self.skipPrevIterNaNsInDilation:
                c_prev, c_current = prevIter_PointForDilation.c(), PointForDilation.c()
                h_prev, h_current = prevIter_PointForDilation.h(), PointForDilation.h()
                
            """                                             Handling switch to NaN                                            """
            NaN_derivatives_excluded = False
            if self.skipPrevIterNaNsInDilation:
                assert self.approach == 'all active'
                
                if not prevIter_PointForDilation.isFeas(True):
                    """                          processing NaNs in nonlin inequality constraints                          """
                    ind_switch_ineq_to_nan = where(logical_and(isnan(c_current), c_prev>0))[0]              
                    if len(ind_switch_ineq_to_nan) != 0:
                        NaN_derivatives_excluded = True
                        tmp = prevIter_PointForDilation.dc(ind_switch_ineq_to_nan)
                        if hasattr(tmp, 'toarray'):
                            tmp = tmp.A
                        if len(ind_switch_ineq_to_nan)>1:
                            tmp *= (c_prev[ind_switch_ineq_to_nan] /sqrt((tmp**2).sum(1))).reshape(-1, 1)
                        else:
                            tmp *= c_prev[ind_switch_ineq_to_nan] / norm(tmp)
                        if tmp.ndim>1: tmp = tmp.sum(0)
                        if not isinstance(tmp, ndarray) or isinstance(tmp, matrix): tmp = tmp.A.flatten() # dense or sparse matrix
                        #print '1: excluded:', norm(tmp), norm(prevDirectionForDilation)
                        prevDirectionForDilation -= tmp
                        #print '1: result=', norm(prevDirectionForDilation)
                        
                    """                           processing NaNs in nonlin equality constraints                           """
                    ind_switch_eq_to_nan = where(logical_and(isnan(h_current), h_prev>0))[0]       
                    if len(ind_switch_eq_to_nan) != 0:
                        NaN_derivatives_excluded = True
                        tmp = prevIter_PointForDilation.dh(ind_switch_eq_to_nan)                        
                        if tmp.ndim>1: tmp = tmp.sum(0)
                        if not isinstance(tmp, ndarray) or isinstance(tmp, matrix): tmp = tmp.A.flatten() # dense or sparse matrix
                        prevDirectionForDilation -= tmp

                    ind_switch_eq_to_nan = where(logical_and(isnan(h_current), h_prev<0))[0]                
                    if len(ind_switch_eq_to_nan) != 0:
                        NaN_derivatives_excluded = True
                        tmp = prevIter_PointForDilation.dh(ind_switch_eq_to_nan)
                        if tmp.ndim>1: tmp = tmp.sum(0)
                        if not isinstance(tmp, ndarray) or isinstance(tmp, matrix): tmp = tmp.A.flatten() # dense or sparse matrix
                        prevDirectionForDilation += tmp
                
            directionForDilation = PointForDilation._getDirection(self.approach) 
            
            """                                            Handling switch from NaN                                           """
            if self.skipPrevIterNaNsInDilation:
                
                if not PointForDilation.isFeas(True):
                    
                    """                          processing NaNs in nonlin inequality constraints                          """
                    ind_switch_ineq_from_nan = where(logical_and(isnan(c_prev), c_current>0))[0]
                    if len(ind_switch_ineq_from_nan) != 0:
                        NaN_derivatives_excluded = True
                        tmp = PointForDilation.dc(ind_switch_ineq_from_nan)
                        if hasattr(tmp, 'toarray'):
                            tmp = tmp.A                        
                        if len(ind_switch_ineq_from_nan)>1:
                            tmp *= (c_current[ind_switch_ineq_from_nan] /sqrt((tmp**2).sum(1))).reshape(-1, 1)
                        else:
                            tmp *= c_current[ind_switch_ineq_from_nan] / norm(tmp)                        
                        if tmp.ndim>1: tmp = tmp.sum(0)
                        if not isinstance(tmp, ndarray) or isinstance(tmp, matrix): tmp = tmp.A.flatten() # dense or sparse matrix
                        #print '2: excluded:', norm(tmp), norm(directionForDilation)
                        directionForDilation -= tmp
                        #print '2: result=', norm(directionForDilation)
                        
                        
                    """                           processing NaNs in nonlin equality constraints                           """
                    ind_switch_eq_from_nan = where(logical_and(isnan(h_prev), h_current>0))[0]
                    if len(ind_switch_eq_from_nan) != 0:
                        NaN_derivatives_excluded = True
                        tmp = PointForDilation.dh(ind_switch_eq_from_nan)
                        if tmp.ndim>1: tmp = tmp.sum(0)
                        if not isinstance(tmp, ndarray) or isinstance(tmp, matrix): tmp = tmp.A.flatten() # dense or sparse matrix
                        directionForDilation -= tmp

                    ind_switch_eq_from_nan = where(logical_and(isnan(h_prev), h_current<0))[0]
                    if len(ind_switch_eq_from_nan) != 0:
                        NaN_derivatives_excluded = True
                        tmp = PointForDilation.dh(ind_switch_eq_from_nan)
                        if tmp.ndim>1: tmp = tmp.sum(0)
                        if not isinstance(tmp, ndarray) or isinstance(tmp, matrix): tmp = tmp.A.flatten() # dense or sparse matrix
                        directionForDilation += tmp

#            # CHANGES
#            gn = g2/norm(g2)
#            if len(directionVectorsList) == 0 or n < 3: pass
#            else:
#                if len(directionVectorsList) == 1 or abs(dot(directionVectorsList[-1], directionVectorsList[-2]))>0.999:
#                    projectionComponentLenght = abs(dot(directionVectorsList[-1], gn))
#                    restLength = sqrt(1 - min((1, projectionComponentLenght))**2)
#                else: 
#                    e1 = directionVectorsList[-1]
#                    e2 = directionVectorsList[-2] - dot(directionVectorsList[-1], directionVectorsList[-2]) * directionVectorsList[-1]
#                    e2 /= norm(e2)
#                   
#                    proj1, proj2 = dot(e1, gn), dot(e2, gn)
#                    rest = gn - proj1 * e1 - proj2 * e2
#                    restLength = norm(rest)
#                if restLength > 1+1e-5: p.pWarn('possible error in ralg solver: incorrect restLength, exceeds 1.0')
#                
#                # TODO: make it parameters of ralg
#                commonCoeff, alp_add_coeff = 0.5, 1.0
#                
#                if restLength < commonCoeff * (n - 2.0) / n:
#                    #pass
#                    alpAddition = 0.5+(arctan((n - 2.0) / (n * restLength)) - pi / 4.0) / (pi / 2.0) * alp_add_coeff
#                    #p.debugmsg('alpAddition:' + str(alpAddition))
#                    assert alpAddition > 0 # if someone incorrectly modifies commonCoeff it can be less than zero
#                    alp_addition += alpAddition
#                    #p.debugmsg('alp_addition:' + str(alp_addition))
#                    
#            directionVectorsList.append(gn)
#            if len(directionVectorsList) > 2: directionVectorsList = directionVectorsList[:-2]
#            # CHANGES END

                
            if self.dilationType == 'normalized' and (not fname_p in ('lb', 'ub', 'lin_eq', 'lin_ineq') \
                                                      or not fname_ in ('lb', 'ub', 'lin_eq', 'lin_ineq')) and (fname_p != fname_  or ind_p != ind_):
                G2,  G = directionForDilation/norm(directionForDilation), prevDirectionForDilation/norm(prevDirectionForDilation)
            else:
                G2,  G = directionForDilation, prevDirectionForDilation            
           
            if prevIter_PointForDilation.isFeas(True) == PointForDilation.isFeas(True):
                g1 = G2 - G
            elif prevIter_PointForDilation.isFeas(True):
                g1 = G2.copy()
            else:
                g1 = G.copy()
                alp_addition += 0.05
                
            #print p.getMaxResidual(PointForDilation.x, 1)
            ##############################################
            # the case may be occured when 
            #  1) lastPointOfSameType is used 
            # or
            #  2) some NaN from constraints have been excluded
            if norm(G2 - G) < 1e-12 * min((norm(G2), norm(G))) and (involve_lastPointOfSameType or NaN_derivatives_excluded):
                p.debugmsg("ralg: 'last point of same type gradient' is used")
                g1 = G2
            ##############################################


                #g1 = -G.copy() # signum doesn't matter here


            # changes wrt infeas constraints
#            if prevIterPoint.nNaNs() != 0:
#                cp, hp = prevIterPoint.c(), prevIterPoint.h()
#                ind_infeas_cp, ind_infeas_hp = isnan(cp), isnan(hp)
#                
#                c, h = iterPoint.c(), iterPoint.h()
#                ind_infeas_c, ind_infeas_h = isnan(c), isnan(h)
#                
#                ind_goodChange_c = logical_and(ind_infeas_cp,  logical_not(ind_infeas_c))
#                ind_goodChange_h = logical_and(ind_infeas_hp,  logical_not(ind_infeas_h))
#                
#                any_c, any_h = any(ind_goodChange_c), any(ind_goodChange_h)
#                altDilation = zeros(n)
#                if any_c:
#                    altDilation += sum(atleast_2d(iterPoint.dc(where(ind_goodChange_c)[0])), 0)
#                    assert not any(isnan(altDilation))
#                if any_h:
#                    altDilation += sum(atleast_2d(iterPoint.dh(where(ind_goodChange_h)[0])), 0)
#                if any_c or any_h:
#                    #print '!>', altDilation
#                    #g1 = altDilation
#                    pass
            # changes end


            """                             Perform dilation                               """

            # CHANGES
#            g = economyMult(b.T, g1)
#            gn = g/norm(g)
#            if len(directionVectorsList) == 0 or n < 3 or norm(g1) < 1e-20: pass
#            else:
#                if len(directionVectorsList) == 1 or abs(dot(directionVectorsList[-1], directionVectorsList[-2]))>0.999:
#                    projectionComponentLenght = abs(dot(directionVectorsList[-1], gn))
#                    restLength = sqrt(1 - min((1, projectionComponentLenght))**2)
#                else: 
#                    e1 = directionVectorsList[-1]
#                    e2 = directionVectorsList[-2] - dot(directionVectorsList[-1], directionVectorsList[-2]) * directionVectorsList[-1]
#                    print dot(directionVectorsList[-1], directionVectorsList[-2])
#                    e2 /= norm(e2)
#                    proj1, proj2 = dot(e1, gn), dot(e2, gn)
#                    rest = gn - proj1 * e1 - proj2 * e2
#                    restLength = norm(rest)
#                assert restLength < 1+1e-5, 'error in ralg solver: incorrect restLength'
#                
#                # TODO: make it parameters of ralg
#                commonCoeff, alp_add_coeff = 0.5, 1.0
#                
#                if restLength < commonCoeff * (n - 2.0) / n:
#                    #pass
#                    alpAddition = 0.5+(arctan((n - 2.0) / (n * restLength)) - pi / 4.0) / (pi / 2.0) * alp_add_coeff
#                    #p.debugmsg('alpAddition:' + str(alpAddition))
#                    assert alpAddition > 0 # if someone incorrectly modifies commonCoeff it can be less than zero
#                    alp_addition += alpAddition
#                    #p.debugmsg('alp_addition:' + str(alp_addition))
#                    
#            directionVectorsList.append(gn)
#            if len(directionVectorsList) > 2: directionVectorsList = directionVectorsList[:-2]
            # CHANGES END

            if doDilation:
                g = economyMult(b.T, g1)
                ng = p.norm(g)

                if self.needRej(p, b, g1, g) or selfNeedRej:
                    selfNeedRej = False
                    if self.showRej or p.debug:
                        p.info('debug msg: matrix B restoration in ralg solver')
                    b = B0.copy()
                    hs = p.norm(prevIter_best_ls_point.x - best_ls_point.x)
                    # TODO: iterPoint = projection(iterPoint,Aeq) if res_Aeq > 0.75*contol

                if ng < 1e-40: 
                    hs *= 0.9
                    p.debugmsg('small dilation direction norm (%e), skipping' % ng)
                if all(isfinite(g)) and ng > 1e-50 and doDilation:
                    g = (g / ng).reshape(-1,1)
                    vec1 = economyMult(b, g).reshape(-1,1)# TODO: remove economyMult, use dot?
                    #if alp_addition != 0: p.debugmsg('alp_addition:' + str(alp_addition))
                    w = asarray(1.0/(alp+alp_addition)-1.0, T) 
                    vec2 = w * g.T
                    b += p.matmult(vec1, vec2)
            

            """                               Call OO iterfcn                                """
            if hasattr(p, '_df'): delattr(p, '_df')
            if best_ls_point.isFeas(False) and hasattr(best_ls_point, '_df'): 
                p._df = best_ls_point.df().copy()           
                
            p.iterfcn(best_ls_point)
            

            """                             Check stop criteria                           """

            cond_same_point = array_equal(best_ls_point.x, prevIter_best_ls_point.x)
            if cond_same_point and not p.istop:
                p.istop = 14
                p.msg = 'X[k-1] and X[k] are same'
                p.stopdict[SMALL_DELTA_X] = True
                restoreProb()
                self.innerState = {'B': b, 'hs': hs}
                return
            
            s2 = 0
            if p.istop and not p.userStop:
                if p.istop not in p.stopdict: p.stopdict[p.istop] = True # it's actual for converters, TODO: fix it
                if SMALL_DF in p.stopdict:
                    if best_ls_point.isFeas(False): s2 = p.istop
                    p.stopdict.pop(SMALL_DF)
                if SMALL_DELTA_F in p.stopdict:
                    # TODO: implement it more properly
                    if best_ls_point.isFeas(False) and prevIter_best_ls_point.f() != best_ls_point.f(): s2 = p.istop
                    p.stopdict.pop(SMALL_DELTA_F)
                if SMALL_DELTA_X in p.stopdict:
                    if best_ls_point.isFeas(False) or not prevIter_best_ls_point.isFeas(False) or cond_same_point: s2 = p.istop
                    p.stopdict.pop(SMALL_DELTA_X)
#                if s2 and (any(isnan(best_ls_point.c())) or any(isnan(best_ls_point.h()))) \
#                and not p.isNaNInConstraintsAllowed\
#                and not cond_same_point:
#                    s2 = 0
                    
                if not s2 and any(p.stopdict.values()):
                    for key,  val in p.stopdict.items():
                        if val == True:
                            s2 = key
                            break
                p.istop = s2
                
                for key,  val in p.stopdict.items():
                    if key < 0 or key in set([FVAL_IS_ENOUGH, USER_DEMAND_STOP, BUTTON_ENOUGH_HAS_BEEN_PRESSED]):
                        p.iterfcn(bestPoint)
                        self.innerState = {'B': b, 'hs': hs}
                        return
            """                                If stop required                                """
            
            if p.istop:
#                if self.needRej(p, b, g1, g) or not feasiblePointWasEncountered:
#                    b = B0.copy()
#                    hs = max((p.norm(prevIter_best_ls_point.x - best_ls_point.x) , 128*p.xtol))
#                    p.istop = 0
#                else:
                    restoreProb()
                    p.iterfcn(bestPoint)
                    #p.istop, p.msg = istop, msg
                    self.innerState = {'B': b, 'hs': hs}
                    return


            """                Some final things for ralg main cycle                """
#            p.debugmsg('new point Aeq residual:'+str(norm(dot(Aeq, iterPoint.x)-beq)))
#            if needProjection and itn!=0:
#                #pass
#                x2 = self.linEqProjection(iterPoint.x, Aeq, beq)
#                p.debugmsg('norm(delta):' + str(norm(iterPoint.x-x2))) 
#                iterPoint = p.point(x2)
#                p.debugmsg('2: new point Aeq residual:'+str(norm(dot(Aeq, iterPoint.x)-beq)))
            #p.hs.append(hs)
            #g = moveDirection.copy()
            
            #prevDirectionForDilation = directionForDilation

            #iterPoint = None
            #doScale = self.new_s and prevIter_PointForDilation.isFeas(True) !=  best_ls_point.isFeas(True)
            #print doScale
            prevIter_best_ls_point = best_ls_point
            prevIter_PointForDilation = best_ls_point
            prevDirectionForDilation = best_ls_point._getDirection(self.approach)
            moveDirection = best_ls_point._getDirection(self.approach)


    def getPrimevalDilationMatrixWRTlinEqConstraints(self, p):
        n, Aeq, beq = p.n, p.Aeq, p.beq
        nLinEq = len(p.beq)
        ind_fixed = where(p.lb==p.ub)[0]
        arr=ones(n, dtype=self.T)
        arr[ind_fixed] = 0
        b = diag(arr)
        
        if hasattr(Aeq, 'tocsc'):Aeq = Aeq.tocsc()
        
        for i in range(nLinEq):
            vec = Aeq[i]
            #raise 0
            if hasattr(vec, 'toarray'): vec = vec.toarray().flatten()
            g = economyMult(b.T, vec)
            if not any(g): continue
            #ind_nnz = nonzero(g)[0]
            ng = norm(g)
            g = (g / ng).reshape(-1,1)
            
            vec1 = p.matmult(b, g)# TODO: remove economyMult, use dot?
            vec2 = -g.T
            
            b += p.matmult(vec1, vec2)
            
#            if len(ind_nnz) > 0.7 * g.size:
#                b += p.matmult(vec1, vec2)
#            else:
#                ind_nnz1 = nonzero(vec1)[0]
#                ind_nnz2 = nonzero(vec2)[1]
#                r = dot(vec1[ind_nnz1, :], vec2[:, ind_nnz2])
#                if p.debug: 
#                    assert abs(norm(p.matmult(vec1, vec2).flatten()) - norm(r.flatten())) < 1e-5
#                b[ix_(ind_nnz1, ind_nnz2)] += r
        return b
 
    def linEqProjection(self, x, Aeq, beq):
        # TODO: handle case nbeq = 1 ?
        if hasattr(Aeq, 'toarray'):Aeq = Aeq.toarray()
        AeqT = Aeq.T
        AeqAeqT = dot(Aeq, AeqT)
        Aeqx = dot(Aeq, x)
        AeqT_AeqAeqT_inv_Aeqx = dot(AeqT, ravel(solve(AeqAeqT, Aeqx)))
        AeqT_AeqAeqT_inv_beq = dot(AeqT, ravel(solve(AeqAeqT, beq)))
        xf = x - AeqT_AeqAeqT_inv_Aeqx + AeqT_AeqAeqT_inv_beq
        return xf



from interalgLLR import *
from numpy import inf, prod, all, sum, zeros
#from FuncDesigner.boundsurf import boundsurf

# for PyPy
from openopt.kernel.nonOptMisc import where

def r14IP(p, nlhc, residual, definiteRange, y, e, vv, asdf1, C, CBKPMV, g, nNodes,  \
         frc, fTol, Solutions, varTols, _in, dataType, \
         maxNodes, _s, indTC, xRecord):

    required_sigma = p.ftol * 0.99 # to suppress roundoff effects
    
    m, n = y.shape
    
    ip = func10(y, e, vv)
    ip.dictOfFixedFuncs = p.dictOfFixedFuncs
    ip._dictOfRedirectedFuncs = p._dictOfRedirectedFuncs
    ip.maxDistributionSize = p.maxDistributionSize
    ip.surf_preference = True

    tmp = asdf1.interval(ip, ia_surf_level=2)
#    print(type(tmp))
    if hasattr(tmp, 'resolve'):#type(tmp) == boundsurf:
#            print('b')
        #adjustr4WithDiscreteVariables(wr4, p)
        
        if 1:
        # changes
            val_l, val_u = zeros(2*n*m), zeros(2*n*m)
            
            val_l += tmp.l.c # may be array <- scalar
            val_u += tmp.u.c
            
            for v in tmp.dep:
                ind = p._oovarsIndDict[v]
                ts, te = ip[v]#y[:, ind], e[:, ind]
                A, B = (te**2 + te*ts + ts**2) / 3.0, 0.5 * (te + ts)
                #A, B, C = (te**3 - ts**3) / 3.0, 0.5 * (te**2 - ts**2), te - ts
                b = tmp.l.d.get(v, 0.0)
                val_l += b * B
                if tmp.level == 2:
                    a = tmp.l.d2.get(v, 0.0)
                    val_l += a * A
                
                b = tmp.u.d.get(v, 0.0)
                val_u +=  b * B
                if tmp.level == 2:
                    a = tmp.u.d2.get(v, 0.0)
                    val_u += a * A

            #r20 = val_u - val_l
            #approx_value = 0.5 * (val_l + val_u)
            o, a = val_l, val_u
        # changes end
        else:
            cs = oopoint((v, asarray(0.5*(val[0] + val[1]), dataType)) for v, val in ip.items())
            cs.dictOfFixedFuncs = p.dictOfFixedFuncs
            cs._dictOfRedirectedFuncs = p._dictOfRedirectedFuncs
            cs.maxDistributionSize = p.maxDistributionSize

            o, a = tmp.values(cs)
        
        definiteRange = tmp.definiteRange
    else:
        o, a, definiteRange = tmp.lb, tmp.ub, tmp.definiteRange

    
    if not all(definiteRange):
        p.err('''
        numerical integration with interalg is implemented 
        for definite (real) range only, no NaN values in integrand are allowed''')

    o, a = o.reshape(2*n, m).T, a.reshape(2*n, m).T

    nodes = func11(y, e, None, indTC, None, o, a, _s, p)

    #an = nodes if len(_in) == 0 else hstack((_in, nodes)).tolist()
    an = nodes + _in
    
    if 1: 
        an.sort(key = lambda obj: obj.key, reverse=False)
        #an.sort(key = lambda obj: obj.minres, reverse=False)
    else:
        an.sort(key=lambda obj: obj.volumeResidual, reverse=False)

    ao_diff = array([node.key for node in an])
    volumes = array([node.volume for node in an])
    
    r10 = ao_diff <= 0.95*(required_sigma-p._residual) / (prod(p.ub-p.lb) - p._volume)
    ind = where(r10)[0]
    # TODO: use true_sum
    v = volumes[ind]
    p._F += sum(array([an[i].F for i in ind]) * v)
    residuals = ao_diff[ind] * v
    p._residual += residuals.sum()
    p._volume += v.sum()
    
    #an = asarray(an, object)
    #an = an[where(logical_not(r10))[0]]
    an = [elem for i,  elem in enumerate(an) if not r10[i]]
        
    nNodes.append(len(an))
   
    p.iterfcn(xk=array(nan), fk=p._F, rk = 0)#TODO: change rk to something like p._r0 - p._residual
    if p.istop != 0: 
        ao_diff = array([node.key for node in an])
        volumes = array([node.volume for node in an])
        p._residual += sum(ao_diff * volumes)
        _s = None
 
    #an, g = func9(an, fo, g, 'IP')
    #nn = 1 if asdf1.isUncycled and all(isfinite(a)) and all(isfinite(o)) and p._isOnlyBoxBounded else maxNodes
    #an, g = func5(an, nn, g)

    return an, g, inf, _s, Solutions, xRecord, frc, CBKPMV



#!/usr/bin/python
'''
A simple OpenOpt TSP example for directed multigraph;
requires networkx (http://networkx.lanl.gov)
and FuncDesigner installed.
For some solvers limitations on time, cputime, "enough" value, basic GUI features are available.
See http://openopt.org/TSP for more details
'''
from openopt import *
from numpy import sin, cos#, abs
import networkx as nx

N = 5
G = nx.MultiDiGraph()

G.add_edges_from(\
                 [(i,j,{'time': 1.5*(cos(i)+sin(j)+1)**2, 'cost':(i-j)**2 + 2*sin(i) + 2*cos(j)+1, 'way': 'aircraft'}) for i in range(N) for j in range(N) if i != j ])
G.add_edges_from(\
                 [(i,j,{'time': 4.5*(cos(i)-sin(j)+1)**2, 'cost':(i-j)**2 + sin(i) + cos(j)+1, 'way': 'railroad'}) for i in range(int(2*N/3)) for j in range(int(N)) if i != j ])

objective = 'time'

# for solver sa handling of constraints is unimplemented yet
# when your solver is interalg you can use nonlinear constraints as well
# for nonlinear objective and constraints functions like arctan, abs etc should be imported from FuncDesigner

constraints = lambda values: (2 * values['time'] + 3 * values['cost'] > 100, 8 * values['time'] + 15*values['cost'] > 150)

p = TSP(G, objective = objective, constraints = constraints, startNode = 2, returnToStart=True)

solver = 'glpk' # also you can use some other solvers - sa, interalg, OpenOpt MILP solvers
r = p.solve(solver) 

# if your solver is cplex or interalg, you can provide some stop criterion, 
# e.g. maxTime, maxCPUTime, fEnough etc, for example 
# r = p.solve('cplex', maxTime = 100, fEnough = 10) 
# i.e. stop if solution with 10 nodes has been obtained or 100 sec were elapsed
# (gplk and lpSolve has no iterfcn connected and thus cannot handle those criterions)
# also you can use p.manage() to enable basic GUI (http://openopt.org/OOFrameworkDoc#Solving) 
# it requires tkinter installed, that is included into PythonXY, EPD;
# for linux use [sudo] easy_install tk or [sodo] apt-get install python-tk
# r = p.manage('cplex')

print(r.Edges)
'''
Solver:   Time Elapsed = 0.04 	CPU Time Elapsed = 0.04
objFunValue: 6.0653623 (feasible, MaxResidual = 4.44089e-16)
[(4, 3, {'cost': -1.4935899838167472, 'way': 'aircraft', 'time': 0.35644984211087011}), 
(3, 1, {'cost': 6.3628446278560142, 'way': 'aircraft', 'time': 1.0875234238200695}), 
(1, 2, {'cost': 2.4253241482607542, 'way': 'railroad', 'time': 1.7917522081892421}), 
(2, 0, {'cost': 8.8185948536513639, 'way': 'aircraft', 'time': 0.51132677471086396}), 
(0, 4, {'cost': 15.692712758272776, 'way': 'aircraft', 'time': 2.318310053508891})]
'''



from openopt import NLP
from numpy import cos, arange, ones, asarray, abs, zeros, sqrt, asscalar, inf
from string import rjust, ljust, expandtabs, center, lower
N = 100
M = 5
Power = 1.13
ff = lambda x: (abs(x-M) ** Power).sum()
x0 = cos(arange(N))

c = [lambda x: 2* x[0] **4-32, lambda x: x[1]**2+x[2]**2 - 8]
#c = lambda x: [2* x[0] **4-32, x[1]**2+x[2]**2 - 8000]

h1 = lambda x: 1e1*(x[-1]-1)**4
h2 = lambda x: (x[-2]-1.5)**4

h = (h1, h2)



lb = -6*ones(N)
ub = 6*ones(N)
lb[3] = 5.5
ub[4] = 4.5
gtol=1e-6
ftol = 1e-6
diffInt = 1e-8
contol = 1e-6
maxFunEvals = 1e6
maxTime = 10
Xlabel = 'time'
PLOT = 0
colors = ['b', 'k', 'y', 'g', 'r', 'm', 'c']

###############################################################
solvers = ['ralg', 'scipy_cobyla', 'lincher', 'scipy_slsqp', 'ipopt','algencan']
#solvers = ['ralg', 'ipopt']
#solvers = ['ralg', 'scipy_cobyla', 'lincher', 'scipy_slsqp', 'ipopt']
#solvers = ['ralg', 'ralg3']
solvers = ['ralg']
###############################################################

lines, results = [], {}
for j, solver in enumerate(solvers):
    p = NLP(ff, x0, xlabel = Xlabel, c=c, h=h,  lb = lb, ub = ub, gtol=gtol, diffInt = diffInt, ftol = ftol, maxIter = 1390, plot = PLOT, color = colors[j], iprint = 10, df_iter = 4, legend = solver, show=False,  contol = contol,  maxTime = maxTime,  maxFunEvals = maxFunEvals, name='NLP_5')


    if solver =='algencan':
        p.gtol = 1e-2
    elif solver == 'ralg':
        pass
        #p.debug = 1
    
    p.debug = 1
    
    r = p.solve(solver)
    for fn in ('h','c'):
        if not r.evals.has_key(fn): r.evals[fn]=0 # if no c or h are used in problem
    results[solver] = (r.ff, p.getMaxResidual(r.xf), r.elapsed['solver_time'], r.elapsed['solver_cputime'], r.evals['f'], r.evals['c'], r.evals['h'])
    if PLOT:
        subplot(2,1,1)
        F0 = asscalar(p.f(p.x0))
        lines.append(plot([0, 1e-15], [F0, F0], color= colors[j]))

if PLOT:
    for i in range(2):
        subplot(2,1,i+1)
        legend(lines, solvers)

    subplots_adjust(bottom=0.2, hspace=0.3)

    xl = ['Solver                              f_opt     MaxConstr   Time   CPUTime  fEvals  cEvals  hEvals']

    for i in range(len(results)):
        s=(ljust(lower(solvers[i]), 40-len(solvers[i]))+'%0.3f'% (results[solvers[i]][0]) + '        %0.1e' % (results[solvers[i]][1]) + ('      %0.2f'% (results[solvers[i]][2])) + '     %0.2f      '% (results[solvers[i]][3]) + str(results[solvers[i]][4]) + '   ' + rjust(str(results[solvers[i]][5]), 5) + ' '*8 +str(results[solvers[i]][6]))

        xl.append(s)

    xl = '\n'.join(xl)
    subplot(2,1,1)
    xlabel(Xlabel)

    from pylab import *
    subplot(2,1,2)
    xlabel(xl)
    show()




from baseProblem import NonLinProblem
from numpy import asarray, ones, inf
from setDefaultIterFuncs import MAX_NON_SUCCESS 

class GLP(NonLinProblem):
    probType = 'GLP'
    _optionalData = ['lb', 'ub', 'c', 'A', 'b']
    expectedArgs = ['f', 'x0']
    allowedGoals = ['minimum', 'min', 'maximum', 'max']
    goal = 'minimum'
    showGoal = False
    isObjFunValueASingleNumber = True
    plotOnlyCurrentMinimum= True
    _currentBestPoint = None
    _nonSuccessCounter = 0
    maxNonSuccess = 15
    
    def __init__(self, *args, **kwargs):
        #if len(args) > 1: self.err('incorrect args number for GLP constructor, must be 0..1 + (optionaly) some kwargs')

        NonLinProblem.__init__(self, *args, **kwargs)
        
        def maxNonSuccess(p):
            newPoint = p.point(p.xk)
            if self._currentBestPoint is None:
                self._currentBestPoint = newPoint
                return False
            elif newPoint.betterThan(self._currentBestPoint):
                self._currentBestPoint = newPoint
                self._nonSuccessCounter = 0
                return False
            self._nonSuccessCounter += 1
            if self._nonSuccessCounter > self.maxNonSuccess:
                return (True, 'Non-Success Number > maxNonSuccess = ' + str(self.maxNonSuccess))
            else:
                return False
        
        self.kernelIterFuncs[MAX_NON_SUCCESS] = maxNonSuccess
        if 'lb' in kwargs.keys():
            self.n = len(kwargs['lb'])
        elif 'ub' in kwargs.keys():
            self.n = len(kwargs['ub'])
        if hasattr(self, 'n'):
            if not hasattr(self, 'lb'):
                self.lb = -inf * ones(self.n)
            if not hasattr(self, 'ub'):
                self.ub =  inf * ones(self.n)
            if 'x0' not in kwargs and len(args) < 2: 
                self.x0 = (asarray(self.lb) + asarray(self.ub)) / 2.0



"""
Example:
(x0-5)^2 + (x2-5)^2 + ... +(x149-5)^2 -> min

subjected to

# lb<= x <= ub:
x4 <= 4
8 <= x5 <= 15

# Ax <= b
x0+...+x149 >= 825
x9 + x19 <= 3
x10+x11 <= 9

# Aeq x = beq
x100+x101 = 11

# c(x) <= 0
2*x0^4-32 <= 0
x1^2+x2^2-8 <= 0

# h(x) = 0
(x[149]-1)**6 = 0
(x[148]-1.5)**6 = 0
"""

from openopt import NLP
from numpy import cos, arange, ones, asarray, zeros, mat, array
N = 150

# objective function:
f = lambda x: ((x-5)**2).sum()

# objective function gradient (optional):
df = lambda x: 2*(x-5)

# start point (initial estimation)
x0 = 8*cos(arange(N))

# c(x) <= 0 constraints
c = [lambda x: 2* x[0] **4-32, lambda x: x[1]**2+x[2]**2 - 8]

# dc(x)/dx: non-lin ineq constraints gradients (optional):
dc0 = lambda x: [8 * x[0]**3] + [0]*(N-1)
dc1 = lambda x: [0, 2 * x[1],  2 * x[2]] + [0]*(N-3)
dc = [dc0, dc1]

# h(x) = 0 constraints
def h(x):
    #return (x[-1]-1)**2
    return (x[-1]-1)**6, (x[-2]-1.5)**6
    # other possible return types: numpy array, matrix, Python list, tuple

# dh(x)/dx: non-lin eq constraints gradients (optional):
def dh(x):
    r = zeros((2, N))
    r[0, -1] = 6*(x[-1]-1)**5
    r[1, -2] = 6*(x[-2]-1.5)**5
    return r

#def dh(x):
#    r = zeros((1, N))
#    r[0, -1] = 2*(x[-1]-1)**1
#    #r[1, -2] = 6*(x[148]-1.5)**5
#    return r

# lower and upper bounds on variables
lb = -6*ones(N)
ub = 6*ones(N)
ub[4] = 4
lb[5], ub[5] = 8, 15

#lb[110:120]=5.67
#ub[110:120]=5.67

# general linear inequality constraints
A = zeros((3, N))
A[0, 9] = 1
A[0, 19] = 1
A[1, 10:12] = 1
A[2] = -ones(N)
b = [7, 9, -825]

# general linear equality constraints
Aeq = zeros((1, N))
Aeq[0, 100:102] = 1
beq = 11
#Aeq[1, 105] = 1
#Aeq[1, 106] = 1.00001
#beq = [11, 11.00005]

# required constraints tolerance, default for NLP is 1e-6
contol = 1e-6

# If you use solver algencan, NB! - it ignores xtol and ftol; using maxTime, maxCPUTime, maxIter, maxFunEvals, fEnough is recommended.
# Note that in algencan gtol means norm of projected gradient of  the Augmented Lagrangian
# so it should be something like 1e-3...1e-5
gtol = 1e-7 # (default gtol = 1e-6)

# Assign problem:
# 1st arg - objective function
# 2nd arg - start point

p = NLP(f, x0, df=df,  c=c,  dc=dc, h=h,  dh=dh,  A=A,  b=b,  Aeq=Aeq,  beq=beq,  
        lb=lb, ub=ub, gtol=gtol, contol=contol, iprint = 50, maxIter = 10000, maxFunEvals = 1e7, name = 'NLP_1')

#p = NLP(f, x0, df=df,  Aeq=Aeq, beq=beq, 
#        gtol=gtol, contol=contol, iprint = 50, maxIter = 10000, maxFunEvals = 1e7, name = 'NLP_1')

#p = NLP(f, x0, df=df,  lb=lb, ub=ub, gtol=gtol, contol=contol, iprint = 50, maxIter = 10000, maxFunEvals = 1e7, name = 'NLP_1')

#optional: graphic output, requires pylab (matplotlib)
p.plot = 1

#optional: user-supplied 1st derivatives check
p.checkdf()
p.checkdc()
p.checkdh()


def MyIterFcn(p):
    return 0
#    print 'Iteration',p.iter
#    if p.iter == 50:
#       p.user.mylist.append(p.xk.copy()) 
#    return 0

p.user.mylist = []
# solve the problem
#p.debug=1
solver = 'algencan'
solver = 'ralg'
#solver = 'scipy_cobyla'
#solver = 'ipopt'
#solver = 'scipy_slsqp'

p.debug=1
r = p.solve(solver, showRej=1, iprint=1, maxTime = 15000,  newLinEq=1, callback = MyIterFcn) # string argument is solver name
#r = p.solve('r2', iprint = 1, plot=0, showLS=1, maxIter=480)

# r.xf and r.ff are optim point and optim objFun value
# r.ff should be something like 132.0522 



from NLOPT_AUX import NLOPT_AUX
from NLOPT_BASE import NLOPT_BASE
import nlopt

class mlsl(NLOPT_BASE):
    __name__ = 'mlsl'
    __alg__ = 'Multi-Level Single-Linkage'
    
    __optionalDataThatCanBeHandled__ = ['lb', 'ub']
    #funcForIterFcnConnection = 'f'
    population = 0
    __isIterPointAlwaysFeasible__ = True
    _requiresFiniteBoxBounds = True
    
    def __init__(self): pass
    def __solver__(self, p):
        nlopt_opts = {'set_population':self.population} if self.population != 0 else {}
        #if self.population != 0: p.f_iter = self.population
        #p.f_iter = 4*p.n
        NLOPT_AUX(p, nlopt.G_MLSL_LDS, nlopt_opts)



#from numpy import asfarray, argmax, sign, inf, log10
from openopt.kernel.baseSolver import baseSolver
from openopt.kernel.nonOptMisc import where
#from numpy import asfarray,  inf,  atleast_1d
from openopt.kernel.setDefaultIterFuncs import SMALL_DELTA_X,  SMALL_DELTA_F
import numpy as np
nanPenalty = 1e10

# In PyPy np.random is unimplemented yet
#global asdf
#asdf = 0
try:
    from np import random
    Rand = random.rand
    Seed = random.seed
    Randint = random.randint
except:# ImportError, AttributeError
 
    import random
    Seed = random.seed
    def Rand(*shape):
        
        r = np.empty(np.prod(shape))
        for i in range(r.size):
            r[i] = random.random()
        
#        #change
#        global asdf
#        for i in range(r.size):
#            asdf += 1
#            r[i] = np.sin(asdf)
        
        return r.reshape(shape)
    def Randint(low, high=None, size = None):
        assert high is None, 'unimplemented yet'
        if size is None:
            
            #change
#            global asdf
#            asdf += 1
#            return asdf % low
            
            return random.randint(0, low-1)
        a = np.empty(np.prod(size) if not np.isscalar(size) else size, dtype=int)
        for i in range(a.size):
            a[i] = random.randint(0, low-1)
            
            #change
#            global asdf
#            asdf += 1
#            a[i] = asdf % low
            
            
        return a.reshape(size)


class de(baseSolver):
    __name__ = 'de'
    __license__ = "BSD"
    __authors__ = "Stepan Hlushak, stepanko - at - gmail - dot - com, connected to OO by Dmitrey"
    __alg__ = "Two array differential evolution algorithm, Feoktistov V. Differential Evolution. In Search of Solutions (Springer, 2006)(ISBN 0387368965)."
    iterfcnConnected = True
    __homepage__ = ''
    __isIterPointAlwaysFeasible__ = lambda self, p: p.__isNoMoreThanBoxBounded__()
    __optionalDataThatCanBeHandled__ = ['lb', 'ub', 'A', 'b', 'Aeq','beq','c','h']
    _requiresFiniteBoxBounds = True
    
    """Strategy for selecting base vector.
    'random' - random base vector.
    'best' - base vector is chosen to be the best one.
    """
    baseVectorStrategy = 'random'
    
    """Strategy of evaluating difference vector.
    'random' - random direction.
    'best' - move into direction of best vectors.
    """
    searchDirectionStrategy = 'random'
    
    """Strategy of choosing F factor.
    'constant' - F factor is constant.
    'random' - F is vector of random numbers, that are 
             less or equal than F.
    """
    differenceFactorStrategy = 'random'
    
    population = 'default: 10*nVars will be used' 
    differenceFactor = 0.8
    crossoverRate = 0.5
    hndvi = 1
    seed = 150880
    

    __info__ = """
        This is two array differential evolution algorithm.
        
        Can handle constraints Ax <= b and c(x) <= 0 
        
        Finite box-bound constraints lb <= x <= ub are required.
        
        Parameters:
            population - population number (should be ~ 10*nVars),
            differenceFactor - difference factor (should be in (0,1] range),
            crossoverRate - constant of crossover (should be ~ 0.5, in (0,1] range),
            baseVectorStrategy - strategy of selection of base vector, can 
                                   be 'random' (default) or 'best' (best vector).
            searchDirectionStrategy - strategy for calculation of the difference vector,
                                    can be 'random' (random direction, default) or 
                                    'best' (direction of the best vectors).
            differenceFactorStrategy - strategy for difference factror (F), can be
                                    'constant' (constant factor, default) or
                                    'random' (F is chosen random for each vector and 
                                    each component of vector).
            hndvi -   half of number of individuals that take part in creation
                         of difference vector.
        """

    def __init__(self):pass
    def __solver__(self, p):

        #def DE2arr(func, lb, ub, itermax=1000, NP=100, F=0.8, Cr=0.5, strategy=(1, 1, 1, 1), constraints=None):
        
        if not p.__isFiniteBoxBounded__(): p.err('this solver requires finite lb, ub: lb <= x <= ub')
        
        p.kernelIterFuncs.pop(SMALL_DELTA_X, None)
        p.kernelIterFuncs.pop(SMALL_DELTA_F, None)
        
        lb, ub = p.lb, p.ub
        D = p.n#dimension

        if isinstance(self.population,str):
            NP = 10*D
        else:
            NP = self.population
            
        F = self.differenceFactor
        
        Cr = self.crossoverRate
        
        #func = p.f
        
        #constraints = lambda x: np.hstack((p.c(x), p._get_AX_Less_B_residuals(x)))
        
        #################################################
        Seed(self.seed)
        
        #initialize population
        pop = Rand(NP,D)*(ub-lb) + lb
        
        if np.any(np.isfinite(p.x0)):
            pop[0] = np.copy(p.x0)

        pop = _correct_box_constraints(lb,ub,pop)
        
        #evaluate  population 
        best, vals, constr_vals = _eval_pop(pop, p)

        Best = p.point(best[2], f=best[0], mr = best[1], mrName = None, mrInd = 0)

        if self.baseVectorStrategy == 'random':
            useRandBaseVectStrat = True 
        elif self.baseVectorStrategy == 'best':
            useRandBaseVectStrat = False
        else:
            p.err('incorrect baseVectorStrategy, should be "random" or "best", got ' + str(self.baseVectorStrategy))
        
        if self.searchDirectionStrategy == 'random':
            useRandSearchDirStrat = True
        elif self.searchDirectionStrategy == 'best':
            useRandSearchDirStrat = False
        else:
            p.err('incorrect searchDirectionStrategy, should be "random" or "best", got ' + str(self.searchDirectionStrategy))
            
        if self.differenceFactorStrategy == 'random':
            useRandDiffFactorStrat = True 
        elif self.differenceFactorStrategy == 'constant':
            useRandDiffFactorStrat = False
        else:
            p.err('incorrect differenceFactorStrategy, should be "random" or "constant", got ' + str(self.differenceFactorStrategy))
        
        for i in range(p.maxIter+10):
            
            old_pop = pop
            old_vals = vals
            old_constr_vals = constr_vals
            Old_best = Best

            #BASE VECTOR
            if useRandBaseVectStrat: #random base vector
                try:
                    beta = old_pop[Randint(NP, size=NP)]
                except:
                    beta = np.array([old_pop[Randint(NP)] for j in range(NP)])
            else: #best vector
                beta = np.ones((NP,D),'d')
                beta = beta*best[2] #TODO: I think there is a better way to create such matrix

            num_ind = self.hndvi #half of the number of individuals that take part
            #                      in differential creation
            
            #DIFFERENCE
            if useRandSearchDirStrat: #random search
                r1_ints = Randint(NP, size=(num_ind,NP))
                r2_ints = Randint(NP, size=(num_ind,NP))
                try:
                    barycenter1 = (old_pop[r1_ints]).sum(0)
                    barycenter2 = (old_pop[r2_ints]).sum(0)
                except: # PyPy
                    assert self.hndvi == 1, 'unimplemented for PyPy yet'
                    barycenter1 = np.array([old_pop[j] for j in r1_ints[0]])
                    barycenter2 = np.array([old_pop[j] for j in r2_ints[0]])
            else: #directed search
                r_ints = Randint(NP, size=(2*num_ind))
                list = [ (j,constr_vals[j],vals[j]) for j in r_ints]
                list_arr = np.array(list, dtype=[('i', int),
                                                  ('constr_val', float),
                                                  ('val', float)])
                list_arr.sort(order=['constr_val','val'])
                best_list = list_arr[0:num_ind]
                best_arr = np.array([j for (j,c,f) in best_list], 'i')
                worst_list = list_arr[num_ind:2*num_ind]
                worst_arr = np.array([j for (j,c,f) in worst_list], 'i')
                
                try:
                    barycenter1 = ((old_pop[worst_arr]).sum(0))/num_ind
                    barycenter2 = ((old_pop[best_arr]).sum(0))/num_ind
                except: # PyPy
                    assert self.hndvi == 1, 'unimplemented for PyPy yet'
                    barycenter1 = np.array([old_pop[j] for j in worst_arr[0]])
                    barycenter2 = np.array([old_pop[j] for j in best_arr[0]])

            if num_ind != 1:
                barycenter1 /= num_ind
                barycenter2 /= num_ind

            delta = barycenter2 - barycenter1 #should be (NP,D)-shape array

            if useRandDiffFactorStrat:
                Ft = Rand(NP,D)
                Ft = Ft*F
            else:
                Ft = F
            
            pop = beta + Ft*delta
            
            #CROSSOVER
            cross_v = np.ones((NP,D))
            const_v = Rand(NP,D)

            const_v = np.ceil(const_v - Cr) 
            cross_v = cross_v - const_v
            
            pop = old_pop*const_v + pop*cross_v

            #CHECK CONSTRAINTS
            
            pop = _correct_box_constraints(lb,ub,pop)
            
            best, vals, constr_vals = _eval_pop(pop, p)
            Best = p.point(best[2], f=best[0], mr = best[1], mrName = None, mrInd = 0)
            
            #SELECTION
            bool_constr_v = old_constr_vals < constr_vals #true when the old individual is better
            bool_v = old_vals < vals #also true when the old individual is better

            # Stephan, I dislike all these integer operations, I'm sure bool operations are faster and clearer, but it's your right // Dmitrey
            zero_constr_v =  1*((constr_vals > 0) + (old_constr_vals > 0))  
            # 0 - when all constrains in old and new pops are satisfied
            # 1 - when not

            bool_constr_v = (bool_constr_v*4 - 2)*zero_constr_v
            bool_v = bool_v*2 - 1
            
            bool_v = bool_constr_v + bool_v
            
            bool_v = bool_v > 0 
            old_sel_v = (bool_v*1).reshape(NP,1) #converting from bool to int array
            sel_v = np.ones((NP,1)) - old_sel_v
            
            pop = old_pop*old_sel_v + pop*sel_v
            
            old_sel_v = old_sel_v.reshape(NP)
            sel_v = sel_v.reshape(NP)
            vals = old_vals*old_sel_v + vals*sel_v
            constr_vals = old_constr_vals*old_sel_v + constr_vals*sel_v
            #END SELECTION
            
            if Old_best.betterThan(Best):
                Best = Old_best
                
            p.iterfcn(Best)
            if p.istop: 
                return

def _eval_pop(pop, p):
    
    NP = pop.shape[0]
    
    constr_vals = np.zeros(NP)
    vals = p.f(pop).flatten()
    
    if vals.size == 1:
        vals = np.array([vals]*NP)
    vals[np.isnan(vals)] = np.inf

    if p.__isNoMoreThanBoxBounded__():
        best_i = vals.argmin()        
        best = (vals[best_i], 0, pop[best_i])
    else:
#        new = 1
#        
#        if new:# and p.isFDmodel:
            # TODO: handle nanPenalty * newPoint.nNaNs()
            #vals = np.empty(pop.shape[0])
            #vals.fill(np.nan)
        
        P = p.point(pop)

        constr_vals = P.mr(checkBoxBounds = False) + nanPenalty * P.nNaNs()

        ind = constr_vals < p.contol

        if not np.any(ind):
            j = np.argmin(constr_vals)
            bestPoint = p.point(pop[j])
            #bestPoint.i = j
        else:
            IND = where(ind)[0]
            #print(IND, pop.shape)
            try:
                P2 = np.atleast_2d(pop[IND])
                F = np.atleast_1d(vals[IND])
            except: # PyPy
                P2 = np.atleast_2d([pop[j] for j in IND])
                F = np.atleast_1d([vals[j] for j in IND])
            J = np.nanargmin(F)
            bestPoint = p.point(P2[J], f=F[J])# TODO: other fields
            #bestPoint.i = IND[J]
            #bestPoint.i = where(ind)[0]

        best = (bestPoint.f(), bestPoint.mr() + nanPenalty * bestPoint.nNaNs(), bestPoint.x)
#    print(vals, constr_vals)
#    from openopt.kernel.ooMisc import isSolved 
#    raise isSolved

    return best, vals, constr_vals
   
    
def _correct_box_constraints(lb, ub, pop):
    diff_lb = pop - lb
    check_lb = diff_lb < 0.0
    scale = 1.0
    while np.any(check_lb):
        check_lb = check_lb*1
        pop = pop - (1+scale)*check_lb*diff_lb
        diff_lb = pop - lb
        check_lb = diff_lb < 0.0 
        scale /= 2

    diff_ub = pop - ub
    check_ub = diff_ub > 0.0 
    scale = 1.0
    while np.any(check_ub):
        check_ub = check_ub*1
        pop = pop - (1+scale)*check_ub*diff_ub
        diff_ub = pop - ub
        check_ub = diff_ub > 0.0 
        scale /= 2

    # temporary fix
    ind = where(pop<lb)
    if ind[0].size:
        pop[ind] = lb[ind[1]]
    ind = where(pop>ub)
    if ind[0].size:
        pop[ind] = ub[ind[1]]

    return pop



#!/usr/bin/python
'''
An OpenOpt BPP (Bin Packing Problem) example;
requires FuncDesigner installed.
For some solvers limitations on time, cputime, "enough" value, basic GUI features are available.
See http://openopt.org/BPP for more details
'''
from openopt import *
from numpy import sin, cos

N = 15

items = [
         {
         'name': 'item %d' % i, # name of item or item type
         'weight': 1.5*(cos(i)+1)**2, 
         'volume': 2*sin(i) + 3, 
         'n':  1 if i < N/3 else 2 if i < 2*N/3 else 3 # optional: number of items of the type, "1" by default
         } 
         for i in range(N)
         ]
bins = {
'volume': 35, # max volume the bin can store
'weight': 30,  # max weight the bin can handle

# Optional: number of available bins or a good estimation of opt value (no less than the value), 
# it can ESSENTIALLY speedup computations
'n': 5
}
# optional:
# we can add some constraints, each one will affect each bin
constraints = lambda values: 2*values['volume']+0.5*values['weight'] <70
# to add several constraints use constraints = [list_of_constraints],
# e.g. constraints = [lambda v: 2*v['volume']+0.5*v['weight'] <70, lambda v: 4*v['volume']+0.25*v['weight'] >7]
p = BPP(items, bins, constraints = constraints) 
r = p.solve('glpk', iprint = 0) # requires cvxopt and glpk installed, see http://openopt.org/BPP for other solvers
'''------------------------- OpenOpt 0.50 -------------------------
solver: glpk   problem: unnamed    type: MILP   goal: min
 iter   objFunVal   log10(maxResidual)   
    0  0.000e+00               0.48 
    1  4.000e+00            -100.00 
istop: 1000 (optimal)
Solver:   Time Elapsed = 0.11 	CPU Time Elapsed = 0.09
objFunValue: 4 (feasible, MaxResidual = 0)'''

print(r.xf) 
'''[{'item 13': 2, 'item 1': 1, 'item 6': 2}, 
{'item 3': 1, 'item 2': 1, 'item 13': 1, 'item 5': 2, 'item 12': 3}, 
{'item 10': 2, 'item 7': 2, 'item 14': 3}, 
{'item 9': 2, 'item 8': 2, 'item 0': 1, 'item 4': 1, 'item 11': 3, 'item 10': 1}]'''
# pay attention that Python indexation starts from zero: item 0, item 1 ...
# if fields 'name' are absent, you'll have list of numbers instead of Python dict

print(r.values) # per each bin
'''{'volume': (17.245948124126656, 19.886034336769214, 27.3955060854869, 27.004318475761739), 
'weight': (26.000659120389134, 26.211461050259725, 15.120985036297931, 12.972914561409214)}'''



from openopt import NLSP
from numpy import asfarray, zeros, cos, sin, inf

def test(complexity=0, **kwargs):

    f = lambda x: (x[0]**3+x[1]**3-9, x[0]-0.5*x[1], cos(x[2])+x[0]-1.5)

    #optional: gradient
    def df(x):
        df = zeros((3,3))
        df[0,0] = 3*x[0]**2
        df[0,1] = 3*x[1]**2
        df[1,0] = 1
        df[1,1] = -0.5
        df[2,0] = 1
        df[2,2] = -sin(x[2])
        return df

    x0 = [8,15, 80]

    #w/o gradient:
    #p = NLSP(f, x0)

    p = NLSP(f, x0, df = df, maxFunEvals = 1e5, iprint = -1, ftol = 1e-6, contol=1e-35)

    p.lb = [-inf, -inf, 150]
    p.ub = [inf, inf, 158]

    # you could try also comment/uncomment nonlinear constraints:
    p.c = lambda x: (x[2] - 150.8)**2-1.5
    # optional: gradient
    p.dc = lambda x: asfarray((0, 0, 2*(x[2]-150.8)))
    # also you could set it via p=NLSP(f, x0, ..., c = c, dc = dc)

    r = p.solve('nssolve', **kwargs)
    return r.istop>0, r, p


if __name__ == '__main__':
    isPassed, r, p = test()
    
    #assert r.istop > 0
